@@ -17,8 +17,9 @@ Archive Get File
 #include "storage/helper.h"
 
 /**********************************************************************************************************************************/
-ArchiveGetFileResult archiveGetFile(
-    const Storage *storage, const String *request, const List *actualList, const String *walDestination)
+FN_EXTERN ArchiveGetFileResult
+archiveGetFile(
+    const Storage *const storage, const String *const request, const List *const actualList, const String *const walDestination)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE, storage);
@@ -27,6 +28,8 @@ ArchiveGetFileResult archiveGetFile(
         FUNCTION_LOG_PARAM(STRING, walDestination);
     FUNCTION_LOG_END();
 
+    FUNCTION_AUDIT_STRUCT();
+
     ASSERT(request != NULL);
     ASSERT(actualList != NULL && !lstEmpty(actualList));
     ASSERT(walDestination != NULL);
@@ -58,7 +61,7 @@ ArchiveGetFileResult archiveGetFile(
                 {
                     ioFilterGroupAdd(
                         ioWriteFilterGroup(storageWriteIo(destination)),
-                        cipherBlockNew(cipherModeDecrypt, actual->cipherType, BUFSTR(actual->cipherPassArchive), NULL));
+                        cipherBlockNewP(cipherModeDecrypt, actual->cipherType, BUFSTR(actual->cipherPassArchive)));
                     compressible = false;
                 }
 
@@ -87,15 +90,9 @@ ArchiveGetFileResult archiveGetFile(
         // Log errors as warnings and continue
         CATCH_ANY()
         {
-            MEM_CONTEXT_PRIOR_BEGIN()
-            {
-                strLstAdd(
-                    result.warnList,
-                    strNewFmt(
-                        "%s: %s [%s] %s", cfgOptionGroupName(cfgOptGrpRepo, actual->repoIdx), strZ(actual->file),
-                        errorTypeName(errorType()), errorMessage()));
-            }
-            MEM_CONTEXT_PRIOR_END();
+            strLstAddFmt(
+                result.warnList, "%s: %s [%s] %s", cfgOptionGroupName(cfgOptGrpRepo, actual->repoIdx), strZ(actual->file),
+                errorTypeName(errorType()), errorMessage());
         }
         TRY_END();
 
