@@ -10,6 +10,7 @@ Info Command
 
 #include "command/archive/common.h"
 #include "command/info/info.h"
+#include "common/crypto/common.h"
 #include "common/debug.h"
 #include "common/io/fdWrite.h"
 #include "common/lock.h"
@@ -17,7 +18,6 @@ Info Command
 #include "common/memContext.h"
 #include "common/type/json.h"
 #include "config/config.h"
-#include "common/crypto/common.h"
 #include "info/info.h"
 #include "info/infoArchive.h"
 #include "info/infoBackup.h"
@@ -34,6 +34,7 @@ VARIANT_STRDEF_STATIC(ARCHIVE_KEY_MIN_VAR,                          "min");
 VARIANT_STRDEF_STATIC(ARCHIVE_KEY_MAX_VAR,                          "max");
 VARIANT_STRDEF_STATIC(BACKREST_KEY_FORMAT_VAR,                      "format");
 VARIANT_STRDEF_STATIC(BACKREST_KEY_VERSION_VAR,                     "version");
+VARIANT_STRDEF_STATIC(BACKUP_KEY_ANNOTATION_VAR,                    "annotation");
 VARIANT_STRDEF_STATIC(BACKUP_KEY_BACKREST_VAR,                      "backrest");
 VARIANT_STRDEF_STATIC(BACKUP_KEY_ERROR_VAR,                         "error");
 VARIANT_STRDEF_STATIC(BACKUP_KEY_ERROR_LIST_VAR,                    "error-list");
@@ -55,11 +56,13 @@ VARIANT_STRDEF_STATIC(KEY_ARCHIVE_VAR,                              "archive");
 VARIANT_STRDEF_STATIC(KEY_CIPHER_VAR,                               "cipher");
 VARIANT_STRDEF_STATIC(KEY_DATABASE_VAR,                             "database");
 VARIANT_STRDEF_STATIC(KEY_DELTA_VAR,                                "delta");
+VARIANT_STRDEF_STATIC(KEY_DELTA_MAP_VAR,                            "delta-map");
 VARIANT_STRDEF_STATIC(KEY_DESTINATION_VAR,                          "destination");
 VARIANT_STRDEF_STATIC(KEY_NAME_VAR,                                 "name");
 VARIANT_STRDEF_STATIC(KEY_OID_VAR,                                  "oid");
 VARIANT_STRDEF_STATIC(KEY_REPO_KEY_VAR,                             "repo-key");
 VARIANT_STRDEF_STATIC(KEY_SIZE_VAR,                                 "size");
+VARIANT_STRDEF_STATIC(KEY_SIZE_MAP_VAR,                             "size-map");
 VARIANT_STRDEF_STATIC(KEY_START_VAR,                                "start");
 VARIANT_STRDEF_STATIC(KEY_STOP_VAR,                                 "stop");
 VARIANT_STRDEF_STATIC(REPO_KEY_KEY_VAR,                             "key");
@@ -71,6 +74,7 @@ VARIANT_STRDEF_STATIC(STATUS_KEY_CODE_VAR,                          "code");
 VARIANT_STRDEF_STATIC(STATUS_KEY_LOCK_VAR,                          "lock");
 VARIANT_STRDEF_STATIC(STATUS_KEY_LOCK_BACKUP_VAR,                   "backup");
 VARIANT_STRDEF_STATIC(STATUS_KEY_LOCK_BACKUP_HELD_VAR,              "held");
+VARIANT_STRDEF_STATIC(STATUS_KEY_LOCK_BACKUP_PERCENT_COMPLETE_VAR,  "pct-cplt");
 VARIANT_STRDEF_STATIC(STATUS_KEY_MESSAGE_VAR,                       "message");
 
 #define INFO_STANZA_STATUS_OK                                       "ok"
@@ -119,7 +123,7 @@ typedef struct InfoRepoData
 #define FUNCTION_LOG_INFO_REPO_DATA_TYPE                                                                                           \
     InfoRepoData *
 #define FUNCTION_LOG_INFO_REPO_DATA_FORMAT(value, buffer, bufferSize)                                                              \
-    objToLog(value, "InfoRepoData", buffer, bufferSize)
+    objNameToLog(value, "InfoRepoData", buffer, bufferSize)
 
 // Stanza with repository list of information for each repository
 typedef struct InfoStanzaRepo
@@ -129,13 +133,14 @@ typedef struct InfoStanzaRepo
     unsigned int currentPgVersion;                                  // Current postgres version for the stanza
     bool backupLockChecked;                                         // Has the check for a backup lock already been performed?
     bool backupLockHeld;                                            // Is backup lock held on the system where info command is run?
+    const Variant *percentComplete;                                 // Percentage of backup complete * 100 (when not NULL)
     InfoRepoData *repoList;                                         // List of configured repositories
 } InfoStanzaRepo;
 
 #define FUNCTION_LOG_INFO_STANZA_REPO_TYPE                                                                                         \
     InfoStanzaRepo *
 #define FUNCTION_LOG_INFO_STANZA_REPO_FORMAT(value, buffer, bufferSize)                                                            \
-    objToLog(value, "InfoStanzaRepo", buffer, bufferSize)
+    objNameToLog(value, "InfoStanzaRepo", buffer, bufferSize)
 
 // Group all databases with the same system-id and version together regardless of db-id or repo
 typedef struct DbGroup
@@ -151,7 +156,7 @@ typedef struct DbGroup
 #define FUNCTION_LOG_DB_GROUP_TYPE                                                                                                 \
     DbGroup *
 #define FUNCTION_LOG_DB_GROUP_FORMAT(value, buffer, bufferSize)                                                                    \
-    objToLog(value, "DbGroup", buffer, bufferSize)
+    objNameToLog(value, "DbGroup", buffer, bufferSize)
 
 /***********************************************************************************************************************************
 Helper function for reporting errors
@@ -175,15 +180,16 @@ infoStanzaErrorAdd(InfoRepoData *repoList, const ErrorType *type, const String *
 Set the overall error status code and message for the stanza to the code and message passed
 ***********************************************************************************************************************************/
 static void
-stanzaStatus(const int code, bool backupLockHeld, Variant *stanzaInfo)
+stanzaStatus(const int code, const InfoStanzaRepo *const stanzaData, Variant *stanzaInfo)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(INT, code);
-        FUNCTION_TEST_PARAM(BOOL, backupLockHeld);
+        FUNCTION_TEST_PARAM(INFO_STANZA_REPO, stanzaData);
         FUNCTION_TEST_PARAM(VARIANT, stanzaInfo);
     FUNCTION_TEST_END();
 
     ASSERT((code >= 0 && code <= 6) || code == 99);
+    ASSERT(stanzaData != NULL);
     ASSERT(stanzaInfo != NULL);
 
     KeyValue *statusKv = kvPutKv(varKv(stanzaInfo), STANZA_KEY_STATUS_VAR);
@@ -228,7 +234,10 @@ stanzaStatus(const int code, bool backupLockHeld, Variant *stanzaInfo)
     // Construct a specific lock part
     KeyValue *lockKv = kvPutKv(statusKv, STATUS_KEY_LOCK_VAR);
     KeyValue *backupLockKv = kvPutKv(lockKv, STATUS_KEY_LOCK_BACKUP_VAR);
-    kvPut(backupLockKv, STATUS_KEY_LOCK_BACKUP_HELD_VAR, VARBOOL(backupLockHeld));
+    kvPut(backupLockKv, STATUS_KEY_LOCK_BACKUP_HELD_VAR, VARBOOL(stanzaData->backupLockHeld));
+
+    if (stanzaData->percentComplete != NULL && cfgOptionStrId(cfgOptOutput) != CFGOPTVAL_OUTPUT_JSON)
+        kvPut(backupLockKv, STATUS_KEY_LOCK_BACKUP_PERCENT_COMPLETE_VAR, stanzaData->percentComplete);
 
     FUNCTION_TEST_RETURN_VOID();
 }
@@ -299,6 +308,8 @@ archiveDbList(
         FUNCTION_TEST_PARAM(UINT, repoKey);
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(stanza != NULL);
     ASSERT(pgData != NULL);
     ASSERT(archiveSection != NULL);
@@ -390,6 +401,8 @@ backupListAdd(
         FUNCTION_TEST_PARAM(INFO_REPO_DATA, repoData);              // The repo data where this backup is located
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(backupSection != NULL);
     ASSERT(backupData != NULL);
     ASSERT(repoData != NULL);
@@ -398,7 +411,6 @@ backupListAdd(
 
     // Flags used to decide what data to add
     const bool outputJson = cfgOptionStrId(cfgOptOutput) == CFGOPTVAL_OUTPUT_JSON;
-    const bool backupLabelMatch = backupLabel != NULL && strEq(backupData->backupLabel, backupLabel);
 
     // main keys
     kvPut(varKv(backupInfo), BACKUP_KEY_LABEL_VAR, VARSTR(backupData->backupLabel));
@@ -444,6 +456,12 @@ backupListAdd(
     kvPut(repoInfo, KEY_SIZE_VAR, VARUINT64(backupData->backupInfoRepoSize));
     kvPut(repoInfo, KEY_DELTA_VAR, VARUINT64(backupData->backupInfoRepoSizeDelta));
 
+    if (outputJson && backupData->backupInfoRepoSizeMap != NULL)
+    {
+        kvPut(repoInfo, KEY_SIZE_MAP_VAR, backupData->backupInfoRepoSizeMap);
+        kvPut(repoInfo, KEY_DELTA_MAP_VAR, backupData->backupInfoRepoSizeMapDelta);
+    }
+
     // timestamp section
     KeyValue *timeInfo = kvPutKv(varKv(backupInfo), BACKUP_KEY_TIMESTAMP_VAR);
 
@@ -456,15 +474,19 @@ backupListAdd(
         kvPut(varKv(backupInfo), BACKUP_KEY_ERROR_VAR, backupData->backupError);
 
     // Add start/stop backup lsn info to json output or --set text
-    if ((outputJson || backupLabelMatch) && backupData->backupLsnStart != NULL && backupData->backupLsnStop != NULL)
+    if ((outputJson || backupLabel != NULL) && backupData->backupLsnStart != NULL && backupData->backupLsnStop != NULL)
     {
         KeyValue *const lsnInfo = kvPutKv(varKv(backupInfo), BACKUP_KEY_LSN_VAR);
         kvPut(lsnInfo, KEY_START_VAR, VARSTR(backupData->backupLsnStart));
         kvPut(lsnInfo, KEY_STOP_VAR, VARSTR(backupData->backupLsnStop));
     }
 
+    // Add annotations to json output or --set text
+    if ((outputJson || backupLabel != NULL) && backupData->backupAnnotation != NULL)
+        kvPut(varKv(backupInfo), BACKUP_KEY_ANNOTATION_VAR, backupData->backupAnnotation);
+
     // If a backup label was specified and this is that label, then get the data from the loaded manifest
-    if (backupLabelMatch)
+    if (backupLabel != NULL)
     {
         // Get the list of databases in this backup
         VariantList *databaseSection = varLstNew();
@@ -474,7 +496,7 @@ backupListAdd(
             const ManifestDb *db = manifestDb(repoData->manifest, dbIdx);
 
             // Do not display template databases
-            if (db->id > db->lastSystemId)
+            if (!pgDbIsTemplate(db->name))
             {
                 Variant *database = varNewKv(kvNew());
 
@@ -565,8 +587,8 @@ For each current backup in the backup.info file of the stanza, set the data for
 ***********************************************************************************************************************************/
 static void
 backupList(
-    VariantList *backupSection, InfoStanzaRepo *stanzaData, const String *backupLabel, unsigned int repoIdxMin,
-    unsigned int repoIdxMax)
+    VariantList *const backupSection, InfoStanzaRepo *const stanzaData, const String *const backupLabel,
+    const unsigned int repoIdxMin, const unsigned int repoIdxMax)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(VARIANT_LIST, backupSection);           // The section to add the backup data to
@@ -576,6 +598,8 @@ backupList(
         FUNCTION_TEST_PARAM(UINT, repoIdxMax);                      // The index of the last repo to check
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(backupSection != NULL);
     ASSERT(stanzaData != NULL);
 
@@ -621,12 +645,15 @@ backupList(
         InfoRepoData *repoData = &stanzaData->repoList[backupNextRepoIdx];
         InfoBackupData backupData = infoBackupData(repoData->backupInfo, repoData->backupIdx);
         repoData->backupIdx++;
+        backupTotalProcessed++;
+
+        // Don't add the backup data to the backup section if a backup label was specified but this is not it
+        if (backupLabel != NULL && !strEq(backupData.backupLabel, backupLabel))
+            continue;
 
         // Add the backup data to the backup section
         if (!cfgOptionTest(cfgOptType) || cfgOptionStrId(cfgOptType) == backupData.backupType)
             backupListAdd(backupSection, &backupData, backupLabel, repoData);
-
-        backupTotalProcessed++;
     }
 
     FUNCTION_TEST_RETURN_VOID();
@@ -636,7 +663,7 @@ backupList(
 Set the stanza data for each stanza found in the repo
 ***********************************************************************************************************************************/
 static VariantList *
-stanzaInfoList(List *stanzaRepoList, const String *backupLabel, unsigned int repoIdxMin, unsigned int repoIdxMax)
+stanzaInfoList(List *stanzaRepoList, const String *const backupLabel, const unsigned int repoIdxMin, const unsigned int repoIdxMax)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(LIST, stanzaRepoList);
@@ -645,6 +672,8 @@ stanzaInfoList(List *stanzaRepoList, const String *backupLabel, unsigned int rep
         FUNCTION_TEST_PARAM(UINT, repoIdxMax);
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(stanzaRepoList != NULL);
 
     VariantList *result = varLstNew();
@@ -766,7 +795,7 @@ stanzaInfoList(List *stanzaRepoList, const String *backupLabel, unsigned int rep
         kvPut(varKv(stanzaInfo), STANZA_KEY_BACKUP_VAR, varNewVarLst(backupSection));
 
         // Set the overall stanza status
-        stanzaStatus(stanzaStatusCode, stanzaData->backupLockHeld, stanzaInfo);
+        stanzaStatus(stanzaStatusCode, stanzaData, stanzaInfo);
 
         // Set the overall cipher type
         if (stanzaCipherType != INFO_STANZA_STATUS_CODE_MIXED)
@@ -777,7 +806,7 @@ stanzaInfoList(List *stanzaRepoList, const String *backupLabel, unsigned int rep
         varLstAdd(result, stanzaInfo);
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(VARIANT_LIST, result);
 }
 
 /***********************************************************************************************************************************
@@ -791,6 +820,8 @@ formatTextBackup(const DbGroup *dbGroup, String *resultStr)
         FUNCTION_TEST_PARAM(STRING, resultStr);
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(dbGroup != NULL);
 
     strCatFmt(resultStr, "\n        wal archive min/max (%s): ", strZ(dbGroup->version));
@@ -949,6 +980,25 @@ formatTextBackup(const DbGroup *dbGroup, String *resultStr)
             else
                 strCatZ(resultStr, "            error(s) detected during backup\n");
         }
+
+        // Annotations metadata
+        if (kvGet(backupInfo, BACKUP_KEY_ANNOTATION_VAR) != NULL)
+        {
+            const KeyValue *const annotationKv = varKv(kvGet(backupInfo, BACKUP_KEY_ANNOTATION_VAR));
+            const StringList *const annotationKeyList = strLstNewVarLst(kvKeyList(annotationKv));
+            String *const annotationStr = strNew();
+
+            for (unsigned int keyIdx = 0; keyIdx < strLstSize(annotationKeyList); keyIdx++)
+            {
+                const String *const key = strLstGet(annotationKeyList, keyIdx);
+                const String *const value = varStr(kvGet(annotationKv, VARSTR(key)));
+                ASSERT(value != NULL);
+
+                strCatFmt(annotationStr, "                %s: %s\n", strZ(key), strZ(value));
+            }
+
+            strCatFmt(resultStr, "            annotation(s)\n%s", strZ(annotationStr));
+        }
     }
 
     FUNCTION_TEST_RETURN_VOID();
@@ -970,6 +1020,8 @@ formatTextDb(
         FUNCTION_TEST_PARAM(UINT64, currentPgSystemId);
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(stanzaInfo != NULL);
     ASSERT(currentPgVersion != NULL);
 
@@ -1049,11 +1101,6 @@ formatTextDb(
     for (unsigned int backupIdx = 0; backupIdx < varLstSize(backupSection); backupIdx++)
     {
         KeyValue *backupInfo = varKv(varLstGet(backupSection, backupIdx));
-
-        // If a backup label was specified but this is not it then continue
-        if (backupLabel != NULL && !strEq(varStr(kvGet(backupInfo, BACKUP_KEY_LABEL_VAR)), backupLabel))
-            continue;
-
         KeyValue *backupDbInfo = varKv(kvGet(backupInfo, KEY_DATABASE_VAR));
         unsigned int backupDbId = varUInt(kvGet(backupDbInfo, DB_KEY_ID_VAR));
         unsigned int backupRepoKey = varUInt(kvGet(backupDbInfo, KEY_REPO_KEY_VAR));
@@ -1135,6 +1182,8 @@ infoUpdateStanza(
         FUNCTION_TEST_PARAM(STRING, backupLabel);
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(storage != NULL);
     ASSERT(stanzaRepo != NULL);
 
@@ -1189,13 +1238,16 @@ infoUpdateStanza(
                 // If a backup lock check has not already been performed, then do so
                 if (!stanzaRepo->backupLockChecked)
                 {
-                    // Try to acquire a lock. If not possible, assume another backup or expire is already running.
-                    stanzaRepo->backupLockHeld = !lockAcquire(
-                        cfgOptionStr(cfgOptLockPath), stanzaRepo->name, cfgOptionStr(cfgOptExecId), lockTypeBackup, 0, false);
-
-                    // Immediately release the lock acquired
-                    lockRelease(!stanzaRepo->backupLockHeld);
+                    // If there is a valid backup lock for this stanza then backup/expire must be running
+                    stanzaRepo->backupLockHeld = lockRead(
+                        cfgOptionStr(cfgOptLockPath), stanzaRepo->name, lockTypeBackup).status == lockReadStatusValid;
                     stanzaRepo->backupLockChecked = true;
+
+                    if (stanzaRepo->backupLockHeld)
+                    {
+                        stanzaRepo->percentComplete =
+                            lockRead(cfgOptionStr(cfgOptLockPath), stanzaRepo->name, lockTypeBackup).data.percentComplete;
+                    }
                 }
             }
 
@@ -1246,11 +1298,6 @@ infoRender(void)
         const String *backupLabel = cfgOptionStrNull(cfgOptSet);
         bool backupFound = false;
 
-        // Since the --set option depends on the --stanza option, the parser will error before this if the backup label is
-        // specified but a stanza is not
-        if (backupLabel != NULL && cfgOptionStrId(cfgOptOutput) != CFGOPTVAL_OUTPUT_TEXT)
-            THROW(ConfigError, "option '" CFGOPT_SET "' is currently only valid for " CFGOPTVAL_OUTPUT_TEXT_Z " output");
-
         // Initialize the repo index
         unsigned int repoIdxMin = 0;
         unsigned int repoTotal = cfgOptionGroupIdxTotal(cfgOptGrpRepo);
@@ -1431,7 +1478,7 @@ infoRender(void)
         if (cfgOptionStrId(cfgOptOutput) == CFGOPTVAL_OUTPUT_TEXT)
         {
             // Process any stanza directories
-            if  (!varLstEmpty(infoList))
+            if (!varLstEmpty(infoList))
             {
                 for (unsigned int stanzaIdx = 0; stanzaIdx < varLstSize(infoList); stanzaIdx++)
                 {
@@ -1440,7 +1487,7 @@ infoRender(void)
 
                     // Add a carriage return between stanzas
                     if (stanzaIdx > 0)
-                        strCatFmt(resultStr, "\n");
+                        strCatZ(resultStr, "\n");
 
                     // Stanza name and status
                     strCatFmt(resultStr, "stanza: %s\n    status: ", strZ(stanzaName));
@@ -1453,6 +1500,10 @@ infoRender(void)
                     KeyValue *lockKv = varKv(kvGet(stanzaStatus, STATUS_KEY_LOCK_VAR));
                     KeyValue *backupLockKv = varKv(kvGet(lockKv, STATUS_KEY_LOCK_BACKUP_VAR));
                     bool backupLockHeld = varBool(kvGet(backupLockKv, STATUS_KEY_LOCK_BACKUP_HELD_VAR));
+                    const Variant *const percentComplete = kvGet(backupLockKv, STATUS_KEY_LOCK_BACKUP_PERCENT_COMPLETE_VAR);
+                    const String *const percentCompleteStr = percentComplete != NULL ?
+                        strNewFmt(" - %u.%02u%% complete", varUInt(percentComplete) / 100, varUInt(percentComplete) % 100) :
+                        EMPTY_STR;
 
                     if (statusCode != INFO_STANZA_STATUS_CODE_OK)
                     {
@@ -1463,10 +1514,13 @@ infoRender(void)
                             // Stanza status
                             strCatFmt(
                                 resultStr, "%s%s\n",
-                                statusCode == INFO_STANZA_STATUS_CODE_MIXED ? INFO_STANZA_MIXED :
-                                    strZ(strNewFmt(INFO_STANZA_STATUS_ERROR " (%s)",
-                                    strZ(varStr(kvGet(stanzaStatus, STATUS_KEY_MESSAGE_VAR))))),
-                                backupLockHeld == true ? " (" INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP ")" : "");
+                                statusCode == INFO_STANZA_STATUS_CODE_MIXED ?
+                                    INFO_STANZA_MIXED :
+                                    zNewFmt(
+                                        INFO_STANZA_STATUS_ERROR " (%s)",
+                                        strZ(varStr(kvGet(stanzaStatus, STATUS_KEY_MESSAGE_VAR)))),
+                                backupLockHeld == true ?
+                                    zNewFmt(" (" INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP "%s)", strZ(percentCompleteStr)) : "");
 
                             // Output the status per repo
                             VariantList *repoSection = kvGetList(stanzaInfo, STANZA_KEY_REPO_VAR);
@@ -1494,7 +1548,7 @@ infoRender(void)
                                         strCatFmt(
                                             resultStr, "%s%s%s\n",
                                             multiRepo ? INFO_STANZA_STATUS_ERROR " (" INFO_STANZA_STATUS_MESSAGE_OTHER ")\n" : "",
-                                            formatSpacer, strZ(strLstJoin(repoError, strZ(strNewFmt("\n%s", formatSpacer)))));
+                                            formatSpacer, strZ(strLstJoin(repoError, zNewFmt("\n%s", formatSpacer))));
                                     }
                                     else
                                     {
@@ -1511,14 +1565,19 @@ infoRender(void)
                             strCatFmt(
                                 resultStr, "%s (%s%s\n", INFO_STANZA_STATUS_ERROR,
                                 strZ(varStr(kvGet(stanzaStatus, STATUS_KEY_MESSAGE_VAR))),
-                                backupLockHeld == true ? ", " INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP ")" : ")");
+                                backupLockHeld == true ?
+                                    zNewFmt(", " INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP "%s)", strZ(percentCompleteStr)) : ")");
                         }
                     }
                     else
                     {
                         // Change displayed status if backup lock is found
                         if (backupLockHeld)
-                            strCatFmt(resultStr, "%s (%s)\n", INFO_STANZA_STATUS_OK, INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP);
+                        {
+                            strCatFmt(
+                                resultStr, "%s (%s%s)\n", INFO_STANZA_STATUS_OK, INFO_STANZA_STATUS_MESSAGE_LOCK_BACKUP,
+                                strZ(percentCompleteStr));
+                        }
                         else
                             strCatFmt(resultStr, "%s\n", INFO_STANZA_STATUS_OK);
                     }
@@ -1577,7 +1636,7 @@ infoRender(void)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 cmdInfo(void)
 {
     FUNCTION_LOG_VOID(logLevelDebug);
