@@ -44,10 +44,10 @@ The return value should be -1 when item1 < item2, 0 when item1 == item2, and 1 w
 typedef int ListComparator(const void *item1, const void *item2);
 
 // General purpose list comparator for Strings or structs with a String as the first member
-int lstComparatorStr(const void *item1, const void *item2);
+FN_EXTERN int lstComparatorStr(const void *item1, const void *item2);
 
 // General purpose list comparator for zero-terminated strings or structs with a zero-terminated string as the first member
-int lstComparatorZ(const void *item1, const void *item2);
+FN_EXTERN int lstComparatorZ(const void *item1, const void *item2);
 
 /***********************************************************************************************************************************
 Constructors
@@ -62,7 +62,7 @@ typedef struct ListParam
 #define lstNewP(itemSize, ...)                                                                                                     \
     lstNew(itemSize, (ListParam){VAR_PARAM_INIT, __VA_ARGS__})
 
-List *lstNew(size_t itemSize, ListParam param);
+FN_EXTERN List *lstNew(size_t itemSize, ListParam param);
 
 /***********************************************************************************************************************************
 Getters/Setters
@@ -73,24 +73,24 @@ typedef struct ListPub
 } ListPub;
 
 // Set a new comparator
-List *lstComparatorSet(List *this, ListComparator *comparator);
+FN_EXTERN List *lstComparatorSet(List *this, ListComparator *comparator);
 
 // Memory context for this list
-__attribute__((always_inline)) static inline MemContext *
+FN_INLINE_ALWAYS MemContext *
 lstMemContext(List *const this)
 {
     return objMemContext(this);
 }
 
 // List size
-__attribute__((always_inline)) static inline unsigned int
+FN_INLINE_ALWAYS unsigned int
 lstSize(const List *const this)
 {
     return THIS_PUB(List)->listSize;
 }
 
 // Is the list empty?
-__attribute__((always_inline)) static inline bool
+FN_INLINE_ALWAYS bool
 lstEmpty(const List *const this)
 {
     return lstSize(this) == 0;
@@ -100,56 +100,56 @@ lstEmpty(const List *const this)
 Functions
 ***********************************************************************************************************************************/
 // Clear items from a list
-List *lstClear(List *this);
+FN_EXTERN List *lstClear(List *this);
 
 // Get an item from the list
-void *lstGet(const List *this, unsigned int listIdx);
-void *lstGetLast(const List *this);
+FN_EXTERN void *lstGet(const List *this, unsigned int listIdx);
+FN_EXTERN void *lstGetLast(const List *this);
 
 // Find an item in the list
-void *lstFind(const List *this, const void *item);
-void *lstFindDefault(const List *this, const void *item, void *itemDefault);
-unsigned int lstFindIdx(const List *this, const void *item);
+FN_EXTERN void *lstFind(const List *this, const void *item);
+FN_EXTERN void *lstFindDefault(const List *this, const void *item, void *itemDefault);
+FN_EXTERN unsigned int lstFindIdx(const List *this, const void *item);
 
 // Does an item exist in the list?
-__attribute__((always_inline)) static inline bool
+FN_INLINE_ALWAYS bool
 lstExists(const List *const this, const void *const item)
 {
     return lstFind(this, item) != NULL;
 }
 
 // Get the index of a list item
-unsigned int lstIdx(const List *this, const void *item);
+FN_EXTERN unsigned int lstIdx(const List *this, const void *item);
 
 // Insert an item into the list
-void *lstInsert(List *this, unsigned int listIdx, const void *item);
+FN_EXTERN void *lstInsert(List *this, unsigned int listIdx, const void *item);
 
 // Add an item to the end of the list
-__attribute__((always_inline)) static inline void *
+FN_INLINE_ALWAYS void *
 lstAdd(List *const this, const void *const item)
 {
     return lstInsert(this, lstSize(this), item);
 }
 
 // Move to a new parent mem context
-__attribute__((always_inline)) static inline List *
+FN_INLINE_ALWAYS List *
 lstMove(List *const this, MemContext *const parentNew)
 {
     return objMove(this, parentNew);
 }
 
 // Remove an item from the list
-bool lstRemove(List *this, const void *item);
-List *lstRemoveIdx(List *this, unsigned int listIdx);
-List *lstRemoveLast(List *this);
+FN_EXTERN bool lstRemove(List *this, const void *item);
+FN_EXTERN List *lstRemoveIdx(List *this, unsigned int listIdx);
+FN_EXTERN List *lstRemoveLast(List *this);
 
 // List sort
-List *lstSort(List *this, SortOrder sortOrder);
+FN_EXTERN List *lstSort(List *this, SortOrder sortOrder);
 
 /***********************************************************************************************************************************
 Destructor
 ***********************************************************************************************************************************/
-__attribute__((always_inline)) static inline void
+FN_INLINE_ALWAYS void
 lstFree(List *const this)
 {
     objFree(this);
@@ -158,11 +158,11 @@ lstFree(List *const this)
 /***********************************************************************************************************************************
 Macros for function logging
 ***********************************************************************************************************************************/
-String *lstToLog(const List *this);
+FN_EXTERN void lstToLog(const List *this, StringStatic *debugLog);
 
 #define FUNCTION_LOG_LIST_TYPE                                                                                                     \
     List *
 #define FUNCTION_LOG_LIST_FORMAT(value, buffer, bufferSize)                                                                        \
-    FUNCTION_LOG_STRING_OBJECT_FORMAT(value, lstToLog, buffer, bufferSize)
+    FUNCTION_LOG_OBJECT_FORMAT(value, lstToLog, buffer, bufferSize)
 
 #endif
