@@ -10,7 +10,7 @@ PostgreSQL Interface
 #include "common/memContext.h"
 #include "common/regExp.h"
 #include "postgres/interface.h"
-#include "postgres/interface/version.h"
+#include "postgres/interface/static.vendor.h"
 #include "postgres/version.h"
 #include "storage/helper.h"
 
@@ -51,9 +51,7 @@ sectors. This is true across all versions of PostgreSQL.
 /***********************************************************************************************************************************
 PostgreSQL interface definitions
 
-Each supported version of PostgreSQL must have interface files named postgres/interface/vXXX.c/h that implement the functions
-specified in the interface structure below.  The functions are documented here rather than in the interface files so that a change
-in wording does not need to be propagated through N source files.
+The interface functions are documented here rather than the auto-generated file.
 ***********************************************************************************************************************************/
 typedef struct PgInterface
 {
@@ -76,132 +74,8 @@ typedef struct PgInterface
     PgWal (*wal)(const unsigned char *);
 } PgInterface;
 
-static const PgInterface pgInterface[] =
-{
-    {
-        .version = PG_VERSION_14,
-
-        .controlIs = pgInterfaceControlIs140,
-        .control = pgInterfaceControl140,
-        .controlVersion = pgInterfaceControlVersion140,
-
-        .walIs = pgInterfaceWalIs140,
-        .wal = pgInterfaceWal140,
-    },
-    {
-        .version = PG_VERSION_13,
-
-        .controlIs = pgInterfaceControlIs130,
-        .control = pgInterfaceControl130,
-        .controlVersion = pgInterfaceControlVersion130,
-
-        .walIs = pgInterfaceWalIs130,
-        .wal = pgInterfaceWal130,
-    },
-    {
-        .version = PG_VERSION_12,
-
-        .controlIs = pgInterfaceControlIs120,
-        .control = pgInterfaceControl120,
-        .controlVersion = pgInterfaceControlVersion120,
-
-        .walIs = pgInterfaceWalIs120,
-        .wal = pgInterfaceWal120,
-    },
-    {
-        .version = PG_VERSION_11,
-
-        .controlIs = pgInterfaceControlIs110,
-        .control = pgInterfaceControl110,
-        .controlVersion = pgInterfaceControlVersion110,
-
-        .walIs = pgInterfaceWalIs110,
-        .wal = pgInterfaceWal110,
-    },
-    {
-        .version = PG_VERSION_10,
-
-        .controlIs = pgInterfaceControlIs100,
-        .control = pgInterfaceControl100,
-        .controlVersion = pgInterfaceControlVersion100,
-
-        .walIs = pgInterfaceWalIs100,
-        .wal = pgInterfaceWal100,
-    },
-    {
-        .version = PG_VERSION_96,
-
-        .controlIs = pgInterfaceControlIs096,
-        .control = pgInterfaceControl096,
-        .controlVersion = pgInterfaceControlVersion096,
-
-        .walIs = pgInterfaceWalIs096,
-        .wal = pgInterfaceWal096,
-    },
-    {
-        .version = PG_VERSION_95,
-
-        .controlIs = pgInterfaceControlIs095,
-        .control = pgInterfaceControl095,
-        .controlVersion = pgInterfaceControlVersion095,
-
-        .walIs = pgInterfaceWalIs095,
-        .wal = pgInterfaceWal095,
-    },
-    {
-        .version = PG_VERSION_94,
-
-        .controlIs = pgInterfaceControlIs094,
-        .control = pgInterfaceControl094,
-        .controlVersion = pgInterfaceControlVersion094,
-
-        .walIs = pgInterfaceWalIs094,
-        .wal = pgInterfaceWal094,
-    },
-    {
-        .version = PG_VERSION_93,
-
-        .controlIs = pgInterfaceControlIs093,
-        .control = pgInterfaceControl093,
-        .controlVersion = pgInterfaceControlVersion093,
-
-        .walIs = pgInterfaceWalIs093,
-        .wal = pgInterfaceWal093,
-    },
-    {
-        .version = PG_VERSION_92,
-
-        .controlIs = pgInterfaceControlIs092,
-        .control = pgInterfaceControl092,
-        .controlVersion = pgInterfaceControlVersion092,
-
-        .walIs = pgInterfaceWalIs092,
-        .wal = pgInterfaceWal092,
-    },
-    {
-        .version = PG_VERSION_91,
-
-        .controlIs = pgInterfaceControlIs091,
-        .control = pgInterfaceControl091,
-        .controlVersion = pgInterfaceControlVersion091,
-
-        .walIs = pgInterfaceWalIs091,
-        .wal = pgInterfaceWal091,
-    },
-    {
-        .version = PG_VERSION_90,
-
-        .controlIs = pgInterfaceControlIs090,
-        .control = pgInterfaceControl090,
-        .controlVersion = pgInterfaceControlVersion090,
-
-        .walIs = pgInterfaceWalIs090,
-        .wal = pgInterfaceWal090,
-    },
-};
-
-// Total PostgreSQL versions in pgInterface
-#define PG_INTERFACE_SIZE                                           (sizeof(pgInterface) / sizeof(PgInterface))
+// Include auto-generated interfaces
+#include "postgres/interface.auto.c.inc"
 
 /***********************************************************************************************************************************
 These pg_control fields are common to all versions of PostgreSQL, so we can use them to generate error messages when the pg_control
@@ -226,7 +100,7 @@ pgInterfaceVersion(unsigned int pgVersion)
 
     const PgInterface *result = NULL;
 
-    for (unsigned int interfaceIdx = 0; interfaceIdx < PG_INTERFACE_SIZE; interfaceIdx++)
+    for (unsigned int interfaceIdx = 0; interfaceIdx < LENGTH_OF(pgInterface); interfaceIdx++)
     {
         if (pgInterface[interfaceIdx].version == pgVersion)
         {
@@ -239,7 +113,40 @@ pgInterfaceVersion(unsigned int pgVersion)
     if (result == NULL)
         THROW_FMT(AssertError, "invalid " PG_NAME " version %u", pgVersion);
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN_TYPE_CONST_P(PgInterface, result);
+}
+
+/**********************************************************************************************************************************/
+FN_EXTERN bool
+pgDbIsTemplate(const String *const name)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, name);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(BOOL, strEqZ(name, "template0") || strEqZ(name, "template1"));
+}
+
+/**********************************************************************************************************************************/
+FN_EXTERN bool
+pgDbIsSystem(const String *const name)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, name);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(BOOL, strEqZ(name, "postgres") || pgDbIsTemplate(name));
+}
+
+/**********************************************************************************************************************************/
+FN_EXTERN bool
+pgDbIsSystemId(const unsigned int id)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(UINT, id);
+    FUNCTION_TEST_END();
+
+    FUNCTION_TEST_RETURN(BOOL, id < FirstNormalObjectId);
 }
 
 /***********************************************************************************************************************************
@@ -276,7 +183,7 @@ pgControlFromBuffer(const Buffer *controlFile)
     // Search for the version of PostgreSQL that uses this control file
     const PgInterface *interface = NULL;
 
-    for (unsigned int interfaceIdx = 0; interfaceIdx < PG_INTERFACE_SIZE; interfaceIdx++)
+    for (unsigned int interfaceIdx = 0; interfaceIdx < LENGTH_OF(pgInterface); interfaceIdx++)
     {
         if (pgInterface[interfaceIdx].controlIs(bufPtrConst(controlFile)))
         {
@@ -311,7 +218,7 @@ pgControlFromBuffer(const Buffer *controlFile)
     FUNCTION_LOG_RETURN(PG_CONTROL, result);
 }
 
-PgControl
+FN_EXTERN PgControl
 pgControlFromFile(const Storage *storage)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -336,14 +243,14 @@ pgControlFromFile(const Storage *storage)
 }
 
 /**********************************************************************************************************************************/
-uint32_t
+FN_EXTERN uint32_t
 pgControlVersion(unsigned int pgVersion)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgInterfaceVersion(pgVersion)->controlVersion());
+    FUNCTION_TEST_RETURN(UINT32, pgInterfaceVersion(pgVersion)->controlVersion());
 }
 
 /***********************************************************************************************************************************
@@ -359,7 +266,7 @@ typedef struct PgWalCommon
 #define PG_WAL_LONG_HEADER                                          0x0002
 
 /**********************************************************************************************************************************/
-PgWal
+FN_EXTERN PgWal
 pgWalFromBuffer(const Buffer *walBuffer)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
@@ -375,7 +282,7 @@ pgWalFromBuffer(const Buffer *walBuffer)
     // Search for the version of PostgreSQL that uses this WAL magic
     const PgInterface *interface = NULL;
 
-    for (unsigned int interfaceIdx = 0; interfaceIdx < PG_INTERFACE_SIZE; interfaceIdx++)
+    for (unsigned int interfaceIdx = 0; interfaceIdx < LENGTH_OF(pgInterface); interfaceIdx++)
     {
         if (pgInterface[interfaceIdx].walIs(bufPtrConst(walBuffer)))
         {
@@ -404,7 +311,7 @@ pgWalFromBuffer(const Buffer *walBuffer)
     FUNCTION_LOG_RETURN(PG_WAL, result);
 }
 
-PgWal
+FN_EXTERN PgWal
 pgWalFromFile(const String *walFile, const Storage *storage)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -428,7 +335,7 @@ pgWalFromFile(const String *walFile, const Storage *storage)
 }
 
 /**********************************************************************************************************************************/
-String *
+FN_EXTERN String *
 pgTablespaceId(unsigned int pgVersion, unsigned int pgCatalogVersion)
 {
     FUNCTION_TEST_BEGIN();
@@ -450,11 +357,11 @@ pgTablespaceId(unsigned int pgVersion, unsigned int pgCatalogVersion)
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 /**********************************************************************************************************************************/
-uint64_t
+FN_EXTERN uint64_t
 pgLsnFromStr(const String *lsn)
 {
     FUNCTION_TEST_BEGIN();
@@ -473,21 +380,21 @@ pgLsnFromStr(const String *lsn)
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(UINT64, result);
 }
 
-String *
+FN_EXTERN String *
 pgLsnToStr(uint64_t lsn)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(UINT64, lsn);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(strNewFmt("%x/%x", (unsigned int)(lsn >> 32), (unsigned int)(lsn & 0xFFFFFFFF)));
+    FUNCTION_TEST_RETURN(STRING, strNewFmt("%x/%x", (unsigned int)(lsn >> 32), (unsigned int)(lsn & 0xFFFFFFFF)));
 }
 
 /**********************************************************************************************************************************/
-String *
+FN_EXTERN String *
 pgLsnToWalSegment(uint32_t timeline, uint64_t lsn, unsigned int walSegmentSize)
 {
     FUNCTION_TEST_BEGIN();
@@ -497,28 +404,11 @@ pgLsnToWalSegment(uint32_t timeline, uint64_t lsn, unsigned int walSegmentSize)
     FUNCTION_TEST_END();
 
     FUNCTION_TEST_RETURN(
-        strNewFmt("%08X%08X%08X", timeline, (unsigned int)(lsn >> 32), (unsigned int)(lsn & 0xFFFFFFFF) / walSegmentSize));
-}
-
-uint64_t
-pgLsnFromWalSegment(const String *walSegment, unsigned int walSegmentSize)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(STRING, walSegment);
-        FUNCTION_TEST_PARAM(UINT, walSegmentSize);
-    FUNCTION_TEST_END();
-
-    ASSERT(walSegment != NULL);
-    ASSERT(strSize(walSegment) == 24);
-    ASSERT(walSegmentSize > 0);
-
-    FUNCTION_TEST_RETURN(
-        (cvtZToUInt64Base(strZ(strSubN(walSegment, 8, 8)), 16) << 32) +
-        (cvtZToUInt64Base(strZ(strSubN(walSegment, 16, 8)), 16) * walSegmentSize));
+        STRING, strNewFmt("%08X%08X%08X", timeline, (unsigned int)(lsn >> 32), (unsigned int)(lsn & 0xFFFFFFFF) / walSegmentSize));
 }
 
 /**********************************************************************************************************************************/
-uint32_t
+FN_EXTERN uint32_t
 pgTimelineFromWalSegment(const String *const walSegment)
 {
     FUNCTION_TEST_BEGIN();
@@ -528,183 +418,154 @@ pgTimelineFromWalSegment(const String *const walSegment)
     ASSERT(walSegment != NULL);
     ASSERT(strSize(walSegment) == 24);
 
-    char buffer[9];
-
-    strncpy(buffer, strZ(walSegment), sizeof(buffer) - 1);
-    buffer[sizeof(buffer) - 1] = '\0';
-
-    FUNCTION_TEST_RETURN(cvtZToUIntBase(buffer, 16));
+    FUNCTION_TEST_RETURN(UINT32, cvtZSubNToUIntBase(strZ(walSegment), 0, 8, 16));
 }
 
 /**********************************************************************************************************************************/
-StringList *
+FN_EXTERN StringList *
 pgLsnRangeToWalSegmentList(
-    unsigned int pgVersion, uint32_t timeline, uint64_t lsnStart, uint64_t lsnStop, unsigned int walSegmentSize)
+    const uint32_t timeline, const uint64_t lsnStart, const uint64_t lsnStop, const unsigned int walSegmentSize)
 {
     FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(UINT, pgVersion);
         FUNCTION_TEST_PARAM(UINT, timeline);
         FUNCTION_TEST_PARAM(UINT64, lsnStart);
         FUNCTION_TEST_PARAM(UINT64, lsnStop);
         FUNCTION_TEST_PARAM(UINT, walSegmentSize);
     FUNCTION_TEST_END();
 
-    ASSERT(pgVersion != 0);
     ASSERT(timeline != 0);
     ASSERT(lsnStart <= lsnStop);
     ASSERT(walSegmentSize != 0);
-    ASSERT(pgVersion > PG_VERSION_92 || walSegmentSize == PG_WAL_SEGMENT_SIZE_DEFAULT);
 
-    StringList *result = NULL;
+    StringList *const result = strLstNew();
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        result = strLstNew();
-
-        // Skip the FF segment when PostgreSQL <= 9.2 (in this case segment size should always be 16MB)
-        bool skipFF = pgVersion <= PG_VERSION_92;
-
         // Calculate the start and stop segments
         unsigned int startMajor = (unsigned int)(lsnStart >> 32);
         unsigned int startMinor = (unsigned int)(lsnStart & 0xFFFFFFFF) / walSegmentSize;
 
-        unsigned int stopMajor = (unsigned int)(lsnStop >> 32);
-        unsigned int stopMinor = (unsigned int)(lsnStop & 0xFFFFFFFF) / walSegmentSize;
+        const unsigned int stopMajor = (unsigned int)(lsnStop >> 32);
+        const unsigned int stopMinor = (unsigned int)(lsnStop & 0xFFFFFFFF) / walSegmentSize;
 
-        unsigned int minorPerMajor = 0xFFFFFFFF / walSegmentSize;
+        const unsigned int minorPerMajor = 0xFFFFFFFF / walSegmentSize;
 
         // Create list
-        strLstAdd(result, strNewFmt("%08X%08X%08X", timeline, startMajor, startMinor));
+        strLstAddFmt(result, "%08X%08X%08X", timeline, startMajor, startMinor);
 
         while (!(startMajor == stopMajor && startMinor == stopMinor))
         {
             startMinor++;
 
-            if ((skipFF && startMinor == 0xFF) || (!skipFF && startMinor > minorPerMajor))
+            if (startMinor > minorPerMajor)
             {
                 startMajor++;
                 startMinor = 0;
             }
 
-            strLstAdd(result, strNewFmt("%08X%08X%08X", timeline, startMajor, startMinor));
+            strLstAddFmt(result, "%08X%08X%08X", timeline, startMajor, startMinor);
         }
-
-        strLstMove(result, memContextPrior());
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING_LIST, result);
 }
 
 /**********************************************************************************************************************************/
-const String *
+FN_EXTERN const String *
 pgLsnName(unsigned int pgVersion)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgVersion >= PG_VERSION_WAL_RENAME ? PG_NAME_LSN_STR : PG_NAME_LOCATION_STR);
+    FUNCTION_TEST_RETURN_CONST(STRING, pgVersion >= PG_VERSION_WAL_RENAME ? PG_NAME_LSN_STR : PG_NAME_LOCATION_STR);
 }
 
 /***********************************************************************************************************************************
 Get WAL name (wal/xlog) for a PostgreSQL version
 ***********************************************************************************************************************************/
-const String *
+FN_EXTERN const String *
 pgWalName(unsigned int pgVersion)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgVersion >= PG_VERSION_WAL_RENAME ? PG_NAME_WAL_STR : PG_NAME_XLOG_STR);
+    FUNCTION_TEST_RETURN_CONST(STRING, pgVersion >= PG_VERSION_WAL_RENAME ? PG_NAME_WAL_STR : PG_NAME_XLOG_STR);
 }
 
 /**********************************************************************************************************************************/
-const String *
+FN_EXTERN const String *
 pgWalPath(unsigned int pgVersion)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgVersion >= PG_VERSION_WAL_RENAME ? PG_PATH_PGWAL_STR : PG_PATH_PGXLOG_STR);
+    FUNCTION_TEST_RETURN_CONST(STRING, pgVersion >= PG_VERSION_WAL_RENAME ? PG_PATH_PGWAL_STR : PG_PATH_PGXLOG_STR);
 }
 
 /**********************************************************************************************************************************/
-const String *
+FN_EXTERN const String *
 pgXactPath(unsigned int pgVersion)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(UINT, pgVersion);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(pgVersion >= PG_VERSION_WAL_RENAME ? PG_PATH_PGXACT_STR : PG_PATH_PGCLOG_STR);
+    FUNCTION_TEST_RETURN_CONST(STRING, pgVersion >= PG_VERSION_WAL_RENAME ? PG_PATH_PGXACT_STR : PG_PATH_PGCLOG_STR);
 }
 
 /**********************************************************************************************************************************/
-unsigned int
-pgVersionFromStr(const String *version)
+FN_EXTERN unsigned int
+pgVersionFromStr(const String *const version)
 {
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(STRING, version);
-    FUNCTION_LOG_END();
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(STRING, version);
+    FUNCTION_TEST_END();
 
     ASSERT(version != NULL);
 
-    unsigned int result = 0;
+    // If format not number.number (9.4) or number only (10) then error. No check for valid/supported PG version is on purpose.
+    if (!regExpMatchOne(STRDEF("^[0-9]+[.]*[0-9]+$"), version))
+        THROW_FMT(AssertError, "version %s format is invalid", strZ(version));
 
-    MEM_CONTEXT_TEMP_BEGIN()
-    {
-        // If format is not number.number (9.4) or number only (10) then error
-        if (!regExpMatchOne(STRDEF("^[0-9]+[.]*[0-9]+$"), version))
-            THROW_FMT(AssertError, "version %s format is invalid", strZ(version));
+    // If there is no dot then only the major version is needed
+    const int idxStart = strChr(version, '.');
 
-        // If there is a dot set the major and minor versions, else just the major
-        int idxStart = strChr(version, '.');
-        unsigned int major;
-        unsigned int minor = 0;
+    if (idxStart == -1)
+        FUNCTION_TEST_RETURN(UINT, cvtZToUInt(strZ(version)) * 10000);
 
-        if (idxStart != -1)
-        {
-            major = cvtZToUInt(strZ(strSubN(version, 0, (size_t)idxStart)));
-            minor = cvtZToUInt(strZ(strSub(version, (size_t)idxStart + 1)));
-        }
-        else
-            major = cvtZToUInt(strZ(version));
-
-        // No check to see if valid/supported PG version is on purpose
-        result = major * 10000 + minor * 100;
-    }
-    MEM_CONTEXT_TEMP_END();
-
-    FUNCTION_LOG_RETURN(UINT, result);
+    // Major and minor version are needed
+    FUNCTION_TEST_RETURN(
+        UINT, cvtZSubNToUInt(strZ(version), 0, (size_t)idxStart) * 10000 + cvtZToUInt(strZ(version) + (size_t)idxStart + 1) * 100);
 }
 
-String *
+FN_EXTERN String *
 pgVersionToStr(unsigned int version)
 {
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(UINT, version);
-    FUNCTION_LOG_END();
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM(UINT, version);
+    FUNCTION_TEST_END();
 
-    String *result = version >= PG_VERSION_10 ?
-        strNewFmt("%u", version / 10000) : strNewFmt("%u.%u", version / 10000, version % 10000 / 100);
-
-    FUNCTION_LOG_RETURN(STRING, result);
+    FUNCTION_TEST_RETURN(
+        STRING,
+        version >= PG_VERSION_10 ?
+            strNewFmt("%u", version / 10000) : strNewFmt("%u.%u", version / 10000, version % 10000 / 100));
 }
 
 /**********************************************************************************************************************************/
-String *
-pgControlToLog(const PgControl *pgControl)
+FN_EXTERN void
+pgControlToLog(const PgControl *const pgControl, StringStatic *const debugLog)
 {
-    return strNewFmt(
-        "{version: %u, systemId: %" PRIu64 ", walSegmentSize: %u, pageChecksum: %s}", pgControl->version, pgControl->systemId,
-        pgControl->walSegmentSize, cvtBoolToConstZ(pgControl->pageChecksum));
+    strStcFmt(
+        debugLog, "{version: %u, systemId: %" PRIu64 ", walSegmentSize: %u, pageChecksum: %s}", pgControl->version,
+        pgControl->systemId, pgControl->walSegmentSize, cvtBoolToConstZ(pgControl->pageChecksum));
 }
 
-String *
-pgWalToLog(const PgWal *pgWal)
+FN_EXTERN void
+pgWalToLog(const PgWal *const pgWal, StringStatic *const debugLog)
 {
-    return strNewFmt("{version: %u, systemId: %" PRIu64 "}", pgWal->version, pgWal->systemId);
+    strStcFmt(debugLog, "{version: %u, systemId: %" PRIu64 "}", pgWal->version, pgWal->systemId);
 }
