@@ -84,7 +84,7 @@ storageHelperContextInit(void)
 }
 
 /**********************************************************************************************************************************/
-void storageHelperInit(const StorageHelper *const helperList)
+FN_EXTERN void storageHelperInit(const StorageHelper *const helperList)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM_P(VOID, helperList);
@@ -96,7 +96,7 @@ void storageHelperInit(const StorageHelper *const helperList)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageHelperDryRunInit(bool dryRun)
 {
     FUNCTION_TEST_VOID();
@@ -133,7 +133,7 @@ storageHelperStanzaInit(const bool stanzaRequired)
 }
 
 /**********************************************************************************************************************************/
-const Storage *
+FN_EXTERN const Storage *
 storageLocal(void)
 {
     FUNCTION_TEST_VOID();
@@ -152,7 +152,7 @@ storageLocal(void)
     FUNCTION_TEST_RETURN_CONST(STORAGE, storageHelper.storageLocal);
 }
 
-const Storage *
+FN_EXTERN const Storage *
 storageLocalWrite(void)
 {
     FUNCTION_TEST_VOID();
@@ -201,7 +201,7 @@ storagePgGet(unsigned int pgIdx, bool write)
 }
 
 /**********************************************************************************************************************************/
-const Storage *
+FN_EXTERN const Storage *
 storagePgIdx(unsigned int pgIdx)
 {
     FUNCTION_TEST_BEGIN();
@@ -225,14 +225,14 @@ storagePgIdx(unsigned int pgIdx)
     FUNCTION_TEST_RETURN_CONST(STORAGE, storageHelper.storagePg[pgIdx]);
 }
 
-const Storage *
+FN_EXTERN const Storage *
 storagePg(void)
 {
     FUNCTION_TEST_VOID();
     FUNCTION_TEST_RETURN_CONST(STORAGE, storagePgIdx(cfgOptionGroupIdxDefault(cfgOptGrpPg)));
 }
 
-const Storage *
+FN_EXTERN const Storage *
 storagePgIdxWrite(unsigned int pgIdx)
 {
     FUNCTION_TEST_BEGIN();
@@ -260,7 +260,7 @@ storagePgIdxWrite(unsigned int pgIdx)
     FUNCTION_TEST_RETURN_CONST(STORAGE, storageHelper.storagePgWrite[pgIdx]);
 }
 
-const Storage *
+FN_EXTERN const Storage *
 storagePgWrite(void)
 {
     FUNCTION_TEST_VOID();
@@ -396,7 +396,7 @@ storageRepoGet(unsigned int repoIdx, bool write)
 }
 
 /**********************************************************************************************************************************/
-const Storage *
+FN_EXTERN const Storage *
 storageRepoIdx(unsigned int repoIdx)
 {
     FUNCTION_TEST_BEGIN();
@@ -428,14 +428,14 @@ storageRepoIdx(unsigned int repoIdx)
     FUNCTION_TEST_RETURN_CONST(STORAGE, storageHelper.storageRepo[repoIdx]);
 }
 
-const Storage *
+FN_EXTERN const Storage *
 storageRepo(void)
 {
     FUNCTION_TEST_VOID();
     FUNCTION_TEST_RETURN_CONST(STORAGE, storageRepoIdx(cfgOptionGroupIdxDefault(cfgOptGrpRepo)));
 }
 
-const Storage *
+FN_EXTERN const Storage *
 storageRepoIdxWrite(unsigned int repoIdx)
 {
     FUNCTION_TEST_BEGIN();
@@ -471,7 +471,7 @@ storageRepoIdxWrite(unsigned int repoIdx)
     FUNCTION_TEST_RETURN_CONST(STORAGE, storageHelper.storageRepoWrite[repoIdx]);
 }
 
-const Storage *
+FN_EXTERN const Storage *
 storageRepoWrite(void)
 {
     FUNCTION_TEST_VOID();
@@ -522,7 +522,7 @@ storageSpoolPathExpression(const String *expression, const String *path)
 }
 
 /**********************************************************************************************************************************/
-const Storage *
+FN_EXTERN const Storage *
 storageSpool(void)
 {
     FUNCTION_TEST_VOID();
@@ -543,7 +543,7 @@ storageSpool(void)
     FUNCTION_TEST_RETURN_CONST(STORAGE, storageHelper.storageSpool);
 }
 
-const Storage *
+FN_EXTERN const Storage *
 storageSpoolWrite(void)
 {
     FUNCTION_TEST_VOID();
@@ -567,17 +567,3 @@ storageSpoolWrite(void)
 
     FUNCTION_TEST_RETURN_CONST(STORAGE, storageHelper.storageSpoolWrite);
 }
-
-/**********************************************************************************************************************************/
-void
-storageHelperFree(void)
-{
-    FUNCTION_TEST_VOID();
-
-    if (storageHelper.memContext != NULL)
-        memContextFree(storageHelper.memContext);
-
-    storageHelper = (struct StorageHelperLocal){.memContext = NULL, .helperList = storageHelper.helperList};
-
-    FUNCTION_TEST_RETURN_VOID();
-}
