@@ -3,18 +3,12 @@ Remote Storage Protocol Handler
 ***********************************************************************************************************************************/
 #include "build.auto.h"
 
-#include "command/backup/pageChecksum.h"
 #include "common/compress/helper.h"
-#include "common/crypto/cipherBlock.h"
-#include "common/crypto/hash.h"
 #include "common/debug.h"
-#include "common/io/filter/sink.h"
-#include "common/io/filter/size.h"
 #include "common/io/io.h"
 #include "common/log.h"
 #include "common/regExp.h"
 #include "common/type/pack.h"
-#include "common/type/json.h"
 #include "config/config.h"
 #include "protocol/helper.h"
 #include "storage/remote/protocol.h"
@@ -28,13 +22,36 @@ static struct
 {
     MemContext *memContext;                                         // Mem context
     void *driver;                                                   // Storage driver used for requests
+
+    const StorageRemoteFilterHandler *filterHandler;                // Filter handler list
+    unsigned int filterHandlerSize;                                 // Filter handler list size
 } storageRemoteProtocolLocal;
 
+/***********************************************************************************************************************************
+Set filter handlers
+***********************************************************************************************************************************/
+FN_EXTERN void
+storageRemoteFilterHandlerSet(const StorageRemoteFilterHandler *filterHandler, unsigned int filterHandlerSize)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_TEST_PARAM_P(VOID, filterHandler);
+        FUNCTION_TEST_PARAM(UINT, filterHandlerSize);
+    FUNCTION_TEST_END();
+
+    ASSERT(filterHandler != NULL);
+    ASSERT(filterHandlerSize > 0);
+
+    storageRemoteProtocolLocal.filterHandler = filterHandler;
+    storageRemoteProtocolLocal.filterHandlerSize = filterHandlerSize;
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
 /***********************************************************************************************************************************
 Set filter group based on passed filters
 ***********************************************************************************************************************************/
 static void
-storageRemoteFilterGroup(IoFilterGroup *filterGroup, const Pack *const filterPack)
+storageRemoteFilterGroup(IoFilterGroup *const filterGroup, const Pack *const filterPack)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(IO_FILTER_GROUP, filterGroup);
@@ -44,52 +61,70 @@ storageRemoteFilterGroup(IoFilterGroup *filterGroup, const Pack *const filterPac
     ASSERT(filterGroup != NULL);
     ASSERT(filterPack != NULL);
 
-    PackRead *const filterList = pckReadNew(filterPack);
-
-    while (!pckReadNullP(filterList))
+    MEM_CONTEXT_TEMP_BEGIN()
     {
-        const StringId filterKey = pckReadStrIdP(filterList);
-        const Pack *const filterParam = pckReadPackP(filterList);
+        PackRead *const filterList = pckReadNew(filterPack);
 
-        IoFilter *filter = compressFilterPack(filterKey, filterParam);
-
-        if (filter != NULL)
-            ioFilterGroupAdd(filterGroup, filter);
-        else
+        while (!pckReadNullP(filterList))
         {
-            switch (filterKey)
+            const StringId filterKey = pckReadStrIdP(filterList);
+            const Pack *const filterParam = pckReadPackP(filterList);
+
+            // If a compression filter
+            IoFilter *filter = compressFilterPack(filterKey, filterParam);
+
+            if (filter != NULL)
             {
-                case CIPHER_BLOCK_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, cipherBlockNewPack(filterParam));
-                    break;
+                ioFilterGroupAdd(filterGroup, filter);
+            }
+            // Else a filter handler
+            else
+            {
+                ASSERT(storageRemoteProtocolLocal.filterHandler != NULL);
 
-                case CRYPTO_HASH_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, cryptoHashNewPack(filterParam));
-                    break;
+                // Search for a filter handler
+                unsigned int filterIdx = 0;
 
-                case PAGE_CHECKSUM_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, pageChecksumNewPack(filterParam));
-                    break;
+                for (; filterIdx < storageRemoteProtocolLocal.filterHandlerSize; filterIdx++)
+                {
+                    // If a match create the filter
+                    if (storageRemoteProtocolLocal.filterHandler[filterIdx].type == filterKey)
+                    {
+                        // Create a filter with parameters
+                        if (storageRemoteProtocolLocal.filterHandler[filterIdx].handlerParam != NULL)
+                        {
+                            ASSERT(filterParam != NULL);
 
-                case SINK_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, ioSinkNew());
-                    break;
+                            ioFilterGroupAdd(
+                                filterGroup, storageRemoteProtocolLocal.filterHandler[filterIdx].handlerParam(filterParam));
+                        }
+                        // Else create a filter without parameters
+                        else
+                        {
+                            ASSERT(storageRemoteProtocolLocal.filterHandler[filterIdx].handlerNoParam != NULL);
+                            ASSERT(filterParam == NULL);
 
-                case SIZE_FILTER_TYPE:
-                    ioFilterGroupAdd(filterGroup, ioSizeNew());
-                    break;
+                            ioFilterGroupAdd(filterGroup, storageRemoteProtocolLocal.filterHandler[filterIdx].handlerNoParam());
+                        }
 
-                default:
+                        // Break on filter match
+                        break;
+                    }
+                }
+
+                // Error when the filter was not found
+                if (filterIdx == storageRemoteProtocolLocal.filterHandlerSize)
                     THROW_FMT(AssertError, "unable to add filter '%s'", strZ(strIdToStr(filterKey)));
             }
         }
     }
+    MEM_CONTEXT_TEMP_END();
 
     FUNCTION_TEST_RETURN_VOID();
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageRemoteFeatureProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -97,6 +132,8 @@ storageRemoteFeatureProtocol(PackRead *const param, ProtocolServer *const server
         FUNCTION_LOG_PARAM(PROTOCOL_SERVER, server);
     FUNCTION_LOG_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(param == NULL);
     ASSERT(server != NULL);
 
@@ -111,7 +148,7 @@ storageRemoteFeatureProtocol(PackRead *const param, ProtocolServer *const server
         {
             MEM_CONTEXT_PRIOR_BEGIN()
             {
-                MEM_CONTEXT_NEW_BEGIN("StorageRemoteProtocol")
+                MEM_CONTEXT_NEW_BEGIN(StorageRemoteProtocol, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
                 {
                     storageRemoteProtocolLocal.memContext = memContextCurrent();
                     storageRemoteProtocolLocal.driver = storageDriver(storage);
@@ -137,7 +174,6 @@ storageRemoteFeatureProtocol(PackRead *const param, ProtocolServer *const server
 /**********************************************************************************************************************************/
 typedef struct StorageRemoteInfoProcotolWriteData
 {
-    MemContext *memContext;                                         // Mem context used to store values from last call
     time_t timeModifiedLast;                                        // timeModified from last call
     mode_t modeLast;                                                // mode from last call
     uid_t userIdLast;                                               // userId from last call
@@ -160,6 +196,8 @@ storageRemoteInfoProtocolPut(
         FUNCTION_TEST_PARAM(STORAGE_INFO, info);
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(data != NULL);
     ASSERT(write != NULL);
     ASSERT(info != NULL);
@@ -205,42 +243,28 @@ storageRemoteInfoProtocolPut(
             pckWriteStrP(write, info->linkDestination);
     }
 
-    // Store defaults to use for the next call. If memContext is NULL this function is only being called one time so there is no
-    // point in storing defaults.
-    if (data->memContext != NULL)
+    // Store defaults to use for the next call
+    data->timeModifiedLast = info->timeModified;
+    data->modeLast = info->mode;
+    data->userIdLast = info->userId;
+    data->groupIdLast = info->groupId;
+
+    if (info->user != NULL && !strEq(info->user, data->user))                                                       // {vm_covered}
     {
-        data->timeModifiedLast = info->timeModified;
-        data->modeLast = info->mode;
-        data->userIdLast = info->userId;
-        data->groupIdLast = info->groupId;
+        strFree(data->user);
+        data->user = strDup(info->user);
+    }
 
-        if (!strEq(info->user, data->user) && info->user != NULL)                                                   // {vm_covered}
-        {
-            strFree(data->user);
-
-            MEM_CONTEXT_BEGIN(data->memContext)
-            {
-                data->user = strDup(info->user);
-            }
-            MEM_CONTEXT_END();
-        }
-
-        if (!strEq(info->group, data->group) && info->group != NULL)                                                // {vm_covered}
-        {
-            strFree(data->group);
-
-            MEM_CONTEXT_BEGIN(data->memContext)
-            {
-                data->group = strDup(info->group);
-            }
-            MEM_CONTEXT_END();
-        }
+    if (info->group != NULL && !strEq(info->group, data->group))                                                    // {vm_covered}
+    {
+        strFree(data->group);
+        data->group = strDup(info->group);
     }
 
     FUNCTION_TEST_RETURN_VOID();
 }
 
-void
+FN_EXTERN void
 storageRemoteInfoProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -277,35 +301,35 @@ storageRemoteInfoProtocol(PackRead *const param, ProtocolServer *const server)
 }
 
 /**********************************************************************************************************************************/
-typedef struct StorageRemoteProtocolInfoListCallbackData
+FN_EXTERN void
+storageRemoteLinkCreateProtocol(PackRead *const param, ProtocolServer *const server)
 {
-    ProtocolServer *const server;
-    StorageRemoteInfoProtocolWriteData writeData;
-} StorageRemoteProtocolInfoListCallbackData;
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(PACK_READ, param);
+        FUNCTION_LOG_PARAM(PROTOCOL_SERVER, server);
+    FUNCTION_LOG_END();
 
-static void
-storageRemoteProtocolInfoListCallback(void *dataVoid, const StorageInfo *info)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_LOG_PARAM_P(VOID, dataVoid);
-        FUNCTION_LOG_PARAM(STORAGE_INFO, info);
-    FUNCTION_TEST_END();
+    ASSERT(param != NULL);
+    ASSERT(server != NULL);
+    ASSERT(storageRemoteProtocolLocal.driver != NULL);
 
-    ASSERT(dataVoid != NULL);
-    ASSERT(info != NULL);
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        const String *const target = pckReadStrP(param);
+        const String *const linkPath = pckReadStrP(param);
+        const StorageLinkType linkType = (StorageLinkType)pckReadU32P(param);
 
-    StorageRemoteProtocolInfoListCallbackData *data = dataVoid;
+        storageInterfaceLinkCreateP(storageRemoteProtocolLocal.driver, target, linkPath, .linkType = linkType);
+        protocolServerDataEndPut(server);
+    }
+    MEM_CONTEXT_TEMP_END();
 
-    PackWrite *write = protocolPackNew();
-    pckWriteStrP(write, info->name);
-    storageRemoteInfoProtocolPut(&data->writeData, write, info);
-    protocolServerDataPut(data->server, write);
-
-    FUNCTION_TEST_RETURN_VOID();
+    FUNCTION_LOG_RETURN_VOID();
 }
 
-void
-storageRemoteInfoListProtocol(PackRead *const param, ProtocolServer *const server)
+/**********************************************************************************************************************************/
+FN_EXTERN void
+storageRemoteListProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(PACK_READ, param);
@@ -320,15 +344,27 @@ storageRemoteInfoListProtocol(PackRead *const param, ProtocolServer *const serve
     {
         const String *const path = pckReadStrP(param);
         const StorageInfoLevel level = (StorageInfoLevel)pckReadU32P(param);
+        StorageRemoteInfoProtocolWriteData writeData = {0};
+        StorageList *const list = storageInterfaceListP(storageRemoteProtocolLocal.driver, path, level);
 
-        StorageRemoteProtocolInfoListCallbackData data = {.server = server, .writeData = {.memContext = memContextCurrent()}};
+        // Put list
+        if (list != NULL)
+        {
+            for (unsigned int listIdx = 0; listIdx < storageLstSize(list); listIdx++)
+            {
+                const StorageInfo info = storageLstGet(list, listIdx);
 
-        const bool result = storageInterfaceInfoListP(
-            storageRemoteProtocolLocal.driver, path, level, storageRemoteProtocolInfoListCallback, &data);
+                PackWrite *const write = protocolPackNew();
+                pckWriteStrP(write, info.name);
+                storageRemoteInfoProtocolPut(&writeData, write, &info);
+                protocolServerDataPut(server, write);
+                pckWriteFree(write);
+            }
+        }
 
         // Indicate whether or not the path was found
         PackWrite *write = protocolPackNew();
-        pckWriteBoolP(write, result, .defaultWrite = true);
+        pckWriteBoolP(write, list != NULL, .defaultWrite = true);
         protocolServerDataPut(server, write);
 
         protocolServerDataEndPut(server);
@@ -339,7 +375,7 @@ storageRemoteInfoListProtocol(PackRead *const param, ProtocolServer *const serve
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageRemoteOpenReadProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -356,7 +392,7 @@ storageRemoteOpenReadProtocol(PackRead *const param, ProtocolServer *const serve
         const String *file = pckReadStrP(param);
         bool ignoreMissing = pckReadBoolP(param);
         const uint64_t offset = pckReadU64P(param);
-        const Variant *limit = jsonToVar(pckReadStrP(param));
+        const Variant *const limit = pckReadNullP(param) ? NULL : VARUINT64(pckReadU64P(param));
         const Pack *const filter = pckReadPackP(param);
 
         // Create the read object
@@ -409,7 +445,7 @@ storageRemoteOpenReadProtocol(PackRead *const param, ProtocolServer *const serve
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageRemoteOpenWriteProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -440,7 +476,7 @@ storageRemoteOpenWriteProtocol(PackRead *const param, ProtocolServer *const serv
             storageInterfaceNewWriteP(
                 storageRemoteProtocolLocal.driver, file, .modeFile = modeFile, .modePath = modePath, .user = user, .group = group,
                 .timeModified = timeModified, .createPath = createPath, .syncFile = syncFile, .syncPath = syncPath,
-                .atomic = atomic));
+                .atomic = atomic, .truncate = true));
 
         // Set filter group based on passed filters
         storageRemoteFilterGroup(ioWriteFilterGroup(fileWrite), filter);
@@ -496,7 +532,7 @@ storageRemoteOpenWriteProtocol(PackRead *const param, ProtocolServer *const serv
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageRemotePathCreateProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -524,7 +560,7 @@ storageRemotePathCreateProtocol(PackRead *const param, ProtocolServer *const ser
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageRemotePathRemoveProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -552,7 +588,7 @@ storageRemotePathRemoveProtocol(PackRead *const param, ProtocolServer *const ser
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageRemotePathSyncProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -577,7 +613,7 @@ storageRemotePathSyncProtocol(PackRead *const param, ProtocolServer *const serve
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageRemoteRemoveProtocol(PackRead *const param, ProtocolServer *const server)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
