@@ -8,10 +8,10 @@ Remote Storage Read
 
 #include "common/compress/helper.h"
 #include "common/debug.h"
+#include "common/io/io.h"
 #include "common/io/read.h"
 #include "common/log.h"
 #include "common/type/convert.h"
-#include "common/type/json.h"
 #include "common/type/object.h"
 #include "storage/remote/protocol.h"
 #include "storage/remote/read.h"
@@ -42,13 +42,14 @@ Macros for function logging
 #define FUNCTION_LOG_STORAGE_READ_REMOTE_TYPE                                                                                      \
     StorageReadRemote *
 #define FUNCTION_LOG_STORAGE_READ_REMOTE_FORMAT(value, buffer, bufferSize)                                                         \
-    objToLog(value, "StorageReadRemote", buffer, bufferSize)
+    objNameToLog(value, "StorageReadRemote", buffer, bufferSize)
 
 /***********************************************************************************************************************************
-Open the file
+Clear protocol if the entire file is not read or an error occurs before the read is complete. This is required to clear the
+protocol state so a subsequent command can succeed.
 ***********************************************************************************************************************************/
-static bool
-storageReadRemoteOpen(THIS_VOID)
+static void
+storageReadRemoteFreeResource(THIS_VOID)
 {
     THIS(StorageReadRemote);
 
@@ -58,47 +59,36 @@ storageReadRemoteOpen(THIS_VOID)
 
     ASSERT(this != NULL);
 
-    bool result = false;
-
-    MEM_CONTEXT_TEMP_BEGIN()
+    // Read if eof has not been reached
+    if (!this->eof)
     {
-        // If the file is compressible add compression filter on the remote
-        if (this->interface.compressible)
+        do
         {
-            ioFilterGroupAdd(
-                ioReadFilterGroup(storageReadIo(this->read)), compressFilter(compressTypeGz, (int)this->interface.compressLevel));
+            MEM_CONTEXT_TEMP_BEGIN()
+            {
+                PackRead *const read = protocolClientDataGet(this->client);
+                pckReadNext(read);
+
+                // If binary then discard
+                if (pckReadType(read) == pckTypeBin)
+                {
+                    pckReadBinP(read);
+                }
+                // Else read is complete so discard the filter list
+                else
+                {
+                    pckReadPackP(read);
+                    protocolClientDataEndGet(this->client);
+
+                    this->eof = true;
+                }
+            }
+            MEM_CONTEXT_TEMP_END();
         }
-
-        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_STORAGE_OPEN_READ);
-        PackWrite *const param = protocolCommandParam(command);
-
-        pckWriteStrP(param, this->interface.name);
-        pckWriteBoolP(param, this->interface.ignoreMissing);
-        pckWriteU64P(param, this->interface.offset);
-        pckWriteStrP(param, jsonFromVar(this->interface.limit));
-        pckWritePackP(param, ioFilterGroupParamAll(ioReadFilterGroup(storageReadIo(this->read))));
-
-        protocolClientCommandPut(this->client, command, false);
-
-        // If the file exists
-        result = pckReadBoolP(protocolClientDataGet(this->client));
-
-        if (result)
-        {
-            // Clear filters since they will be run on the remote side
-            ioFilterGroupClear(ioReadFilterGroup(storageReadIo(this->read)));
-
-            // If the file is compressible add decompression filter locally
-            if (this->interface.compressible)
-                ioFilterGroupAdd(ioReadFilterGroup(storageReadIo(this->read)), decompressFilter(compressTypeGz));
-        }
-        // Else nothing to do
-        else
-            protocolClientDataEndGet(this->client);
+        while (!this->eof);
     }
-    MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(BOOL, result);
+    FUNCTION_LOG_RETURN_VOID();
 }
 
 /***********************************************************************************************************************************
@@ -136,12 +126,12 @@ storageReadRemote(THIS_VOID, Buffer *buffer, bool block)
                     // If binary then read the next block
                     if (pckReadType(read) == pckTypeBin)
                     {
-                        MEM_CONTEXT_BEGIN(THIS_MEM_CONTEXT())
+                        MEM_CONTEXT_OBJ_BEGIN(this)
                         {
                             this->block = pckReadBinP(read);
                             this->remaining = bufUsed(this->block);
                         }
-                        MEM_CONTEXT_END();
+                        MEM_CONTEXT_OBJ_END();
                     }
                     // Else read is complete and get the filter list
                     else
@@ -202,11 +192,97 @@ storageReadRemoteEof(THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->eof);
+    FUNCTION_TEST_RETURN(BOOL, this->eof);
+}
+
+/***********************************************************************************************************************************
+Open the file
+***********************************************************************************************************************************/
+static bool
+storageReadRemoteOpen(THIS_VOID)
+{
+    THIS(StorageReadRemote);
+
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(STORAGE_READ_REMOTE, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+
+    bool result = false;
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        // If the file is compressible add compression filter on the remote
+        if (this->interface.compressible)
+        {
+            ioFilterGroupAdd(
+                ioReadFilterGroup(storageReadIo(this->read)), compressFilter(compressTypeGz, (int)this->interface.compressLevel));
+        }
+
+        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_STORAGE_OPEN_READ);
+        PackWrite *const param = protocolCommandParam(command);
+
+        pckWriteStrP(param, this->interface.name);
+        pckWriteBoolP(param, this->interface.ignoreMissing);
+        pckWriteU64P(param, this->interface.offset);
+
+        if (this->interface.limit == NULL)
+            pckWriteNullP(param);
+        else
+            pckWriteU64P(param, varUInt64(this->interface.limit));
+
+        pckWritePackP(param, ioFilterGroupParamAll(ioReadFilterGroup(storageReadIo(this->read))));
+
+        protocolClientCommandPut(this->client, command, false);
+
+        // If the file exists
+        result = pckReadBoolP(protocolClientDataGet(this->client));
+
+        if (result)
+        {
+            // Clear filters since they will be run on the remote side
+            ioFilterGroupClear(ioReadFilterGroup(storageReadIo(this->read)));
+
+            // If the file is compressible add decompression filter locally
+            if (this->interface.compressible)
+                ioFilterGroupAdd(ioReadFilterGroup(storageReadIo(this->read)), decompressFilter(compressTypeGz));
+
+            // Set free callback to ensure the protocol is cleared on a short read
+            memContextCallbackSet(objMemContext(this), storageReadRemoteFreeResource, this);
+        }
+        // Else nothing to do
+        else
+            protocolClientDataEndGet(this->client);
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN(BOOL, result);
+}
+
+/***********************************************************************************************************************************
+Close the file and read any remaining data. It is possible that all data has been read but if the amount of data is exactly
+divisible by the buffer size then the eof may not have been received.
+***********************************************************************************************************************************/
+static void
+storageReadRemoteClose(THIS_VOID)
+{
+    THIS(StorageReadRemote);
+
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(STORAGE_READ_REMOTE, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+
+    memContextCallbackClear(objMemContext(this));
+    storageReadRemoteFreeResource(this);
+
+    FUNCTION_LOG_RETURN_VOID();
 }
 
 /**********************************************************************************************************************************/
-StorageRead *
+FN_EXTERN StorageRead *
 storageReadRemoteNew(
     StorageRemote *const storage, ProtocolClient *const client, const String *const name, const bool ignoreMissing,
     const bool compressible, const unsigned int compressLevel, const uint64_t offset, const Variant *const limit)
@@ -228,9 +304,9 @@ storageReadRemoteNew(
 
     StorageReadRemote *this = NULL;
 
-    OBJ_NEW_BEGIN(StorageReadRemote)
+    OBJ_NEW_BEGIN(StorageReadRemote, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
-        this = OBJ_NEW_ALLOC();
+        this = OBJ_NAME(OBJ_NEW_ALLOC(), StorageRead::StorageReadRemote);
 
         *this = (StorageReadRemote)
         {
@@ -249,6 +325,7 @@ storageReadRemoteNew(
 
                 .ioInterface = (IoReadInterface)
                 {
+                    .close = storageReadRemoteClose,
                     .eof = storageReadRemoteEof,
                     .open = storageReadRemoteOpen,
                     .read = storageReadRemote,
