@@ -6,12 +6,10 @@ Remote Storage
 #include "common/debug.h"
 #include "common/log.h"
 #include "common/memContext.h"
-#include "common/type/json.h"
 #include "common/type/object.h"
 #include "common/type/pack.h"
 #include "storage/remote/protocol.h"
 #include "storage/remote/read.h"
-#include "storage/remote/storage.intern.h"
 #include "storage/remote/write.h"
 
 /***********************************************************************************************************************************
@@ -46,6 +44,8 @@ storageRemoteInfoGet(StorageRemoteInfoData *const data, PackRead *const read, St
         FUNCTION_TEST_PARAM(STORAGE_INFO, info);
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(data != NULL);
     ASSERT(read != NULL);
     ASSERT(info != NULL);
@@ -91,7 +91,7 @@ storageRemoteInfoGet(StorageRemoteInfoData *const data, PackRead *const read, St
     data->userIdLast = info->userId;
     data->groupIdLast = info->groupId;
 
-    if (!strEq(info->user, data->user) && info->user != NULL)                                                       // {vm_covered}
+    if (info->user != NULL && !strEq(info->user, data->user))                                                       // {vm_covered}
     {
         strFree(data->user);
 
@@ -102,7 +102,7 @@ storageRemoteInfoGet(StorageRemoteInfoData *const data, PackRead *const read, St
         MEM_CONTEXT_END();
     }
 
-    if (!strEq(info->group, data->group) && info->group != NULL)                                                    // {vm_covered}
+    if (info->group != NULL && !strEq(info->group, data->group))                                                    // {vm_covered}
     {
         strFree(data->group);
 
@@ -128,6 +128,8 @@ storageRemoteInfo(THIS_VOID, const String *file, StorageInfoLevel level, Storage
         FUNCTION_LOG_PARAM(BOOL, param.followLink);
     FUNCTION_LOG_END();
 
+    FUNCTION_AUDIT_STRUCT();
+
     ASSERT(this != NULL);
 
     StorageInfo result = {.level = level};
@@ -169,10 +171,42 @@ storageRemoteInfo(THIS_VOID, const String *file, StorageInfoLevel level, Storage
 }
 
 /**********************************************************************************************************************************/
-static bool
-storageRemoteInfoList(
-    THIS_VOID, const String *path, StorageInfoLevel level, StorageInfoListCallback callback, void *callbackData,
-    StorageInterfaceInfoListParam param)
+static void
+storageRemoteLinkCreate(
+    THIS_VOID, const String *const target, const String *const linkPath, const StorageInterfaceLinkCreateParam param)
+{
+    THIS(StorageRemote);
+
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(STORAGE_REMOTE, this);
+        FUNCTION_LOG_PARAM(STRING, target);
+        FUNCTION_LOG_PARAM(STRING, linkPath);
+        FUNCTION_LOG_PARAM(ENUM, param.linkType);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+    ASSERT(target != NULL);
+    ASSERT(linkPath != NULL);
+
+    MEM_CONTEXT_TEMP_BEGIN()
+    {
+        ProtocolCommand *const command = protocolCommandNew(PROTOCOL_COMMAND_STORAGE_LINK_CREATE);
+        PackWrite *const commandParam = protocolCommandParam(command);
+
+        pckWriteStrP(commandParam, target);
+        pckWriteStrP(commandParam, linkPath);
+        pckWriteU32P(commandParam, param.linkType);
+
+        protocolClientExecute(this->client, command, false);
+    }
+    MEM_CONTEXT_TEMP_END();
+
+    FUNCTION_LOG_RETURN_VOID();
+}
+
+/**********************************************************************************************************************************/
+static StorageList *
+storageRemoteList(THIS_VOID, const String *const path, const StorageInfoLevel level, const StorageInterfaceListParam param)
 {
     THIS(StorageRemote);
 
@@ -180,20 +214,17 @@ storageRemoteInfoList(
         FUNCTION_LOG_PARAM(STORAGE_REMOTE, this);
         FUNCTION_LOG_PARAM(STRING, path);
         FUNCTION_LOG_PARAM(ENUM, level);
-        FUNCTION_LOG_PARAM(FUNCTIONP, callback);
-        FUNCTION_LOG_PARAM_P(VOID, callbackData);
         (void)param;                                                // No parameters are used
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(path != NULL);
-    ASSERT(callback != NULL);
 
-    bool result = false;
+    StorageList *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_STORAGE_INFO_LIST);
+        ProtocolCommand *command = protocolCommandNew(PROTOCOL_COMMAND_STORAGE_LIST);
         PackWrite *const commandParam = protocolCommandParam(command);
 
         pckWriteStrP(commandParam, path);
@@ -204,6 +235,7 @@ storageRemoteInfoList(
 
         // Read list
         StorageRemoteInfoData parseData = {.memContext = memContextCurrent()};
+        result = storageLstNew(level);
 
         MEM_CONTEXT_TEMP_RESET_BEGIN()
         {
@@ -215,7 +247,7 @@ storageRemoteInfoList(
                 StorageInfo info = {.exists = true, .level = level, .name = pckReadStrP(read)};
 
                 storageRemoteInfoGet(&parseData, read, &info);
-                callback(callbackData, &info);
+                storageLstAdd(result, &info);
 
                 // Reset the memory context occasionally so we don't use too much memory or slow down processing
                 MEM_CONTEXT_TEMP_RESET(1000);
@@ -224,15 +256,17 @@ storageRemoteInfoList(
                 pckReadNext(read);
             }
 
-            result = pckReadBoolP(read);
+            if (!pckReadBoolP(read))
+                result = NULL;
         }
         MEM_CONTEXT_TEMP_END();
 
         protocolClientDataEndGet(this->client);
+        storageLstMove(result, memContextPrior());
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(BOOL, result);
+    FUNCTION_LOG_RETURN(STORAGE_LIST, result);
 }
 
 /**********************************************************************************************************************************/
@@ -284,6 +318,7 @@ storageRemoteNewWrite(
 
     ASSERT(this != NULL);
     ASSERT(file != NULL);
+    ASSERT(param.truncate);
 
     FUNCTION_LOG_RETURN(
         STORAGE_WRITE,
@@ -423,16 +458,17 @@ storageRemoteRemove(THIS_VOID, const String *file, StorageInterfaceRemoveParam p
 static const StorageInterface storageInterfaceRemote =
 {
     .info = storageRemoteInfo,
-    .infoList = storageRemoteInfoList,
+    .list = storageRemoteList,
     .newRead = storageRemoteNewRead,
     .newWrite = storageRemoteNewWrite,
     .pathCreate = storageRemotePathCreate,
     .pathRemove = storageRemotePathRemove,
     .pathSync = storageRemotePathSync,
     .remove = storageRemoteRemove,
+    .linkCreate = storageRemoteLinkCreate,
 };
 
-Storage *
+FN_EXTERN Storage *
 storageRemoteNew(
     mode_t modeFile, mode_t modePath, bool write, StoragePathExpressionCallback pathExpressionFunction, ProtocolClient *client,
     unsigned int compressLevel)
@@ -452,9 +488,9 @@ storageRemoteNew(
 
     Storage *this = NULL;
 
-    OBJ_NEW_BEGIN(StorageRemote)
+    OBJ_NEW_BEGIN(StorageRemote, .childQty = MEM_CONTEXT_QTY_MAX, .allocQty = MEM_CONTEXT_QTY_MAX)
     {
-        StorageRemote *driver = OBJ_NEW_ALLOC();
+        StorageRemote *const driver = OBJ_NAME(OBJ_NEW_ALLOC(), Storage::StorageRemote);
 
         *driver = (StorageRemote)
         {
