@@ -30,7 +30,7 @@ struct Storage
 };
 
 /**********************************************************************************************************************************/
-Storage *
+FN_EXTERN Storage *
 storageNew(
     StringId type, const String *path, mode_t modeFile, mode_t modePath, bool write,
     StoragePathExpressionCallback pathExpressionFunction, void *driver, StorageInterface interface)
@@ -46,11 +46,13 @@ storageNew(
         FUNCTION_LOG_PARAM(STORAGE_INTERFACE, interface);
     FUNCTION_LOG_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(type != 0);
     ASSERT(strSize(path) >= 1 && strZ(path)[0] == '/');
     ASSERT(driver != NULL);
     ASSERT(interface.info != NULL);
-    ASSERT(interface.infoList != NULL);
+    ASSERT(interface.list != NULL);
     ASSERT(interface.newRead != NULL);
     ASSERT(interface.newWrite != NULL);
     ASSERT(interface.pathRemove != NULL);
@@ -89,11 +91,18 @@ storageNew(
         AssertError, !storageFeature(this, storageFeatureSymLink) || storageFeature(this, storageFeaturePath),
         "path feature required");
 
+    // If link features are enabled then linkCreate must be implemented
+    CHECK(
+        AssertError,
+        (!storageFeature(this, storageFeatureSymLink) && !storageFeature(this, storageFeatureHardLink)) ||
+            interface.linkCreate != NULL,
+        "linkCreate required");
+
     FUNCTION_LOG_RETURN(STORAGE, this);
 }
 
 /**********************************************************************************************************************************/
-bool
+FN_EXTERN bool
 storageCopy(StorageRead *source, StorageWrite *destination)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -115,7 +124,7 @@ storageCopy(StorageRead *source, StorageWrite *destination)
             ioWriteOpen(storageWriteIo(destination));
 
             // Copy data from source to destination
-            ioCopy(storageReadIo(source), storageWriteIo(destination));
+            ioCopyP(storageReadIo(source), storageWriteIo(destination));
 
             // Close the source and destination files
             ioReadClose(storageReadIo(source));
@@ -131,7 +140,7 @@ storageCopy(StorageRead *source, StorageWrite *destination)
 }
 
 /**********************************************************************************************************************************/
-bool
+FN_EXTERN bool
 storageExists(const Storage *this, const String *pathExp, StorageExistsParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -167,7 +176,7 @@ storageExists(const Storage *this, const String *pathExp, StorageExistsParam par
 }
 
 /**********************************************************************************************************************************/
-Buffer *
+FN_EXTERN Buffer *
 storageGet(StorageRead *file, StorageGetParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -224,7 +233,7 @@ storageGet(StorageRead *file, StorageGetParam param)
 }
 
 /**********************************************************************************************************************************/
-StorageInfo
+FN_EXTERN StorageInfo
 storageInfo(const Storage *this, const String *fileExp, StorageInfoParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -236,6 +245,8 @@ storageInfo(const Storage *this, const String *fileExp, StorageInfoParam param)
         FUNCTION_LOG_PARAM(BOOL, param.noPathEnforce);
     FUNCTION_LOG_END();
 
+    FUNCTION_AUDIT_STRUCT();
+
     ASSERT(this != NULL);
     ASSERT(this->pub.interface.info != NULL);
 
@@ -279,185 +290,26 @@ storageInfo(const Storage *this, const String *fileExp, StorageInfoParam param)
 }
 
 /**********************************************************************************************************************************/
-typedef struct StorageInfoListSortData
-{
-    MemContext *memContext;                                         // Mem context to use for allocating data in this struct
-    StringList *ownerList;                                          // List of users and groups to reduce memory usage
-    List *infoList;                                                 // List of info
-} StorageInfoListSortData;
-
-static void
-storageInfoListSortCallback(void *data, const StorageInfo *info)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_LOG_PARAM_P(VOID, data);
-        FUNCTION_LOG_PARAM(STORAGE_INFO, info);
-    FUNCTION_TEST_END();
-
-    StorageInfoListSortData *infoData = data;
-
-    MEM_CONTEXT_BEGIN(infoData->memContext)
-    {
-        // Copy info and dup strings
-        StorageInfo infoCopy = *info;
-        infoCopy.name = strDup(info->name);
-        infoCopy.linkDestination = strDup(info->linkDestination);
-        infoCopy.user = strLstAddIfMissing(infoData->ownerList, info->user);
-        infoCopy.group = strLstAddIfMissing(infoData->ownerList, info->group);
-
-        lstAdd(infoData->infoList, &infoCopy);
-    }
-    MEM_CONTEXT_END();
-
-    FUNCTION_TEST_RETURN_VOID();
-}
-
-static bool
-storageInfoListSort(
-    const Storage *this, const String *path, StorageInfoLevel level, const String *expression, SortOrder sortOrder,
-    StorageInfoListCallback callback, void *callbackData)
-{
-    FUNCTION_LOG_BEGIN(logLevelTrace);
-        FUNCTION_LOG_PARAM(STORAGE, this);
-        FUNCTION_LOG_PARAM(STRING, path);
-        FUNCTION_LOG_PARAM(ENUM, level);
-        FUNCTION_LOG_PARAM(STRING, expression);
-        FUNCTION_LOG_PARAM(ENUM, sortOrder);
-        FUNCTION_LOG_PARAM(FUNCTIONP, callback);
-        FUNCTION_LOG_PARAM_P(VOID, callbackData);
-    FUNCTION_LOG_END();
-
-    ASSERT(this != NULL);
-    ASSERT(callback != NULL);
-
-    bool result = false;
-
-    MEM_CONTEXT_TEMP_BEGIN()
-    {
-        // If no sorting then use the callback directly
-        if (sortOrder == sortOrderNone)
-        {
-            result = storageInterfaceInfoListP(storageDriver(this), path, level, callback, callbackData, .expression = expression);
-        }
-        // Else sort the info before sending it to the callback
-        else
-        {
-            StorageInfoListSortData data =
-            {
-                .memContext = MEM_CONTEXT_TEMP(),
-                .ownerList = strLstNew(),
-                .infoList = lstNewP(sizeof(StorageInfo), .comparator = lstComparatorStr),
-            };
-
-            result = storageInterfaceInfoListP(
-                storageDriver(this), path, level, storageInfoListSortCallback, &data, .expression = expression);
-            lstSort(data.infoList, sortOrder);
-
-            MEM_CONTEXT_TEMP_RESET_BEGIN()
-            {
-                for (unsigned int infoIdx = 0; infoIdx < lstSize(data.infoList); infoIdx++)
-                {
-                    // Pass info to the caller
-                    callback(callbackData, lstGet(data.infoList, infoIdx));
-
-                    // Reset the memory context occasionally
-                    MEM_CONTEXT_TEMP_RESET(1000);
-                }
-            }
-            MEM_CONTEXT_TEMP_END();
-        }
-    }
-    MEM_CONTEXT_TEMP_END();
-
-    FUNCTION_LOG_RETURN(BOOL, result);
-}
-
-typedef struct StorageInfoListData
-{
-    const Storage *storage;                                         // Storage object;
-    StorageInfoListCallback callbackFunction;                       // Original callback function
-    void *callbackData;                                             // Original callback data
-    const String *expression;                                       // Filter for names
-    RegExp *regExp;                                                 // Compiled filter for names
-    bool recurse;                                                   // Should we recurse?
-    SortOrder sortOrder;                                            // Sort order
-    const String *path;                                             // Top-level path for info
-    const String *subPath;                                          // Path below the top-level path (starts as NULL)
-} StorageInfoListData;
-
-static void
-storageInfoListCallback(void *data, const StorageInfo *info)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_LOG_PARAM_P(VOID, data);
-        FUNCTION_LOG_PARAM(STORAGE_INFO, info);
-    FUNCTION_TEST_END();
-
-    StorageInfoListData *listData = data;
-
-    // Is this the . path?
-    bool dotPath = info->type == storageTypePath && strEq(info->name, DOT_STR);
-
-    // Skip . paths when getting info for subpaths (since info was already reported in the parent path)
-    if (dotPath && listData->subPath != NULL)
-    {
-        FUNCTION_TEST_RETURN_VOID();
-        return;
-    }
-
-    // Update the name in info with the subpath
-    StorageInfo infoUpdate = *info;
-
-    if (listData->subPath != NULL)
-        infoUpdate.name = strNewFmt("%s/%s", strZ(listData->subPath), strZ(infoUpdate.name));
-
-    // Is this file a match?
-    bool match = listData->expression == NULL || regExpMatch(listData->regExp, infoUpdate.name);
-
-    // Callback before checking path contents when not descending
-    if (match && listData->sortOrder != sortOrderDesc)
-        listData->callbackFunction(listData->callbackData, &infoUpdate);
-
-    // Recurse into paths
-    if (infoUpdate.type == storageTypePath && listData->recurse && !dotPath)
-    {
-        StorageInfoListData data = *listData;
-        data.subPath = infoUpdate.name;
-
-        storageInfoListSort(
-            data.storage, strNewFmt("%s/%s", strZ(data.path), strZ(data.subPath)), infoUpdate.level, data.expression,
-            data.sortOrder, storageInfoListCallback, &data);
-    }
-
-    // Callback after checking path contents when descending
-    if (match && listData->sortOrder == sortOrderDesc)
-        listData->callbackFunction(listData->callbackData, &infoUpdate);
-
-    FUNCTION_TEST_RETURN_VOID();
-}
-
-bool
-storageInfoList(
-    const Storage *this, const String *pathExp, StorageInfoListCallback callback, void *callbackData, StorageInfoListParam param)
+FN_EXTERN StorageIterator *
+storageNewItr(const Storage *const this, const String *const pathExp, StorageNewItrParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE, this);
         FUNCTION_LOG_PARAM(STRING, pathExp);
-        FUNCTION_LOG_PARAM(FUNCTIONP, callback);
-        FUNCTION_LOG_PARAM_P(VOID, callbackData);
         FUNCTION_LOG_PARAM(ENUM, param.level);
         FUNCTION_LOG_PARAM(BOOL, param.errorOnMissing);
+        FUNCTION_LOG_PARAM(BOOL, param.recurse);
+        FUNCTION_LOG_PARAM(BOOL, param.nullOnMissing);
         FUNCTION_LOG_PARAM(ENUM, param.sortOrder);
         FUNCTION_LOG_PARAM(STRING, param.expression);
         FUNCTION_LOG_PARAM(BOOL, param.recurse);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
-    ASSERT(callback != NULL);
-    ASSERT(this->pub.interface.infoList != NULL);
+    ASSERT(this->pub.interface.list != NULL);
     ASSERT(!param.errorOnMissing || storageFeature(this, storageFeaturePath));
 
-    bool result = false;
+    StorageIterator *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
@@ -465,62 +317,48 @@ storageInfoList(
         if (param.level == storageInfoLevelDefault)
             param.level = storageFeature(this, storageFeatureInfoDetail) ? storageInfoLevelDetail : storageInfoLevelBasic;
 
-        // Build the path
-        String *path = storagePathP(this, pathExp);
-
-        // If there is an expression or recursion then the info will need to be filtered through a local callback
-        if (param.expression != NULL || param.recurse)
-        {
-            StorageInfoListData data =
-            {
-                .storage = this,
-                .callbackFunction = callback,
-                .callbackData = callbackData,
-                .expression = param.expression,
-                .sortOrder = param.sortOrder,
-                .recurse = param.recurse,
-                .path = path,
-            };
-
-            if (data.expression != NULL)
-                data.regExp = regExpNew(param.expression);
-
-            result = storageInfoListSort(
-                this, path, param.level, param.expression, param.sortOrder, storageInfoListCallback, &data);
-        }
-        else
-            result = storageInfoListSort(this, path, param.level, NULL, param.sortOrder, callback, callbackData);
-
-        if (!result && param.errorOnMissing)
-            THROW_FMT(PathMissingError, STORAGE_ERROR_LIST_INFO_MISSING, strZ(path));
+        result = storageItrMove(
+            storageItrNew(
+                storageDriver(this), storagePathP(this, pathExp), param.level, param.errorOnMissing, param.nullOnMissing,
+                param.recurse, param.sortOrder, param.expression),
+            memContextPrior());
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(BOOL, result);
+    FUNCTION_LOG_RETURN(STORAGE_ITERATOR, result);
 }
 
 /**********************************************************************************************************************************/
-static void
-storageListCallback(void *data, const StorageInfo *info)
+FN_EXTERN void storageLinkCreate(
+    const Storage *const this, const String *const target, const String *const linkPath, const StorageLinkCreateParam param)
 {
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_LOG_PARAM_P(VOID, data);
-        FUNCTION_LOG_PARAM(STORAGE_INFO, info);
-    FUNCTION_TEST_END();
+    FUNCTION_LOG_BEGIN(logLevelDebug);
+        FUNCTION_LOG_PARAM(STORAGE, this);
+        FUNCTION_LOG_PARAM(STRING, target);
+        FUNCTION_LOG_PARAM(STRING, linkPath);
+        FUNCTION_LOG_PARAM(ENUM, param.linkType);
+    FUNCTION_LOG_END();
 
-    // Skip . path
-    if (strEq(info->name, DOT_STR))
+    ASSERT(this != NULL);
+    ASSERT(this->write);
+    ASSERT(target != NULL);
+    ASSERT(linkPath != NULL);
+    ASSERT(this->pub.interface.linkCreate != NULL);
+    ASSERT(
+        (param.linkType == storageLinkSym && storageFeature(this, storageFeatureSymLink)) ||
+        (param.linkType == storageLinkHard && storageFeature(this, storageFeatureHardLink)));
+
+    MEM_CONTEXT_TEMP_BEGIN()
     {
-        FUNCTION_TEST_RETURN_VOID();
-        return;
+        storageInterfaceLinkCreateP(storageDriver(this), target, linkPath, .linkType = param.linkType);
     }
+    MEM_CONTEXT_TEMP_END();
 
-    strLstAdd((StringList *)data, info->name);
-
-    FUNCTION_TEST_RETURN_VOID();
+    FUNCTION_LOG_RETURN_VOID();
 }
 
-StringList *
+/**********************************************************************************************************************************/
+FN_EXTERN StringList *
 storageList(const Storage *this, const String *pathExp, StorageListParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -533,26 +371,24 @@ storageList(const Storage *this, const String *pathExp, StorageListParam param)
 
     ASSERT(this != NULL);
     ASSERT(!param.errorOnMissing || !param.nullOnMissing);
-    ASSERT(!param.errorOnMissing || storageFeature(this, storageFeaturePath));
 
     StringList *result = NULL;
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        result = strLstNew();
+        StorageIterator *const storageItr = storageNewItrP(
+            this, pathExp, .errorOnMissing = param.errorOnMissing, .nullOnMissing = param.nullOnMissing,
+            .expression = param.expression);
 
-        // Build an empty list if the directory does not exist by default.  This makes the logic in calling functions simpler when
-        // the caller doesn't care if the path is missing.
-        if (!storageInfoListP(
-                this, pathExp, storageListCallback, result, .level = storageInfoLevelExists, .errorOnMissing = param.errorOnMissing,
-                .expression = param.expression))
+        if (storageItr != NULL)
         {
-            if (param.nullOnMissing)
-                result = NULL;
-        }
+            result = strLstNew();
 
-        // Move list up to the old context
-        result = strLstMove(result, memContextPrior());
+            while (storageItrMore(storageItr))
+                strLstAdd(result, storageItrNext(storageItr).name);
+
+            strLstMove(result, memContextPrior());
+        }
     }
     MEM_CONTEXT_TEMP_END();
 
@@ -560,7 +396,7 @@ storageList(const Storage *this, const String *pathExp, StorageListParam param)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageMove(const Storage *this, StorageRead *source, StorageWrite *destination)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -600,7 +436,7 @@ storageMove(const Storage *this, StorageRead *source, StorageWrite *destination)
 }
 
 /**********************************************************************************************************************************/
-StorageRead *
+FN_EXTERN StorageRead *
 storageNewRead(const Storage *this, const String *fileExp, StorageNewReadParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -631,7 +467,7 @@ storageNewRead(const Storage *this, const String *fileExp, StorageNewReadParam p
 }
 
 /**********************************************************************************************************************************/
-StorageWrite *
+FN_EXTERN StorageWrite *
 storageNewWrite(const Storage *this, const String *fileExp, StorageNewWriteParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -646,11 +482,14 @@ storageNewWrite(const Storage *this, const String *fileExp, StorageNewWriteParam
         FUNCTION_LOG_PARAM(BOOL, param.noSyncFile);
         FUNCTION_LOG_PARAM(BOOL, param.noSyncPath);
         FUNCTION_LOG_PARAM(BOOL, param.noAtomic);
+        FUNCTION_LOG_PARAM(BOOL, param.noTruncate);
         FUNCTION_LOG_PARAM(BOOL, param.compressible);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
     ASSERT(this->write);
+    // noTruncate does not work with atomic writes because a new file is always created for atomic writes
+    ASSERT(!param.noTruncate || param.noAtomic);
 
     StorageWrite *result = NULL;
 
@@ -661,7 +500,8 @@ storageNewWrite(const Storage *this, const String *fileExp, StorageNewWriteParam
                 storageDriver(this), storagePathP(this, fileExp), .modeFile = param.modeFile != 0 ? param.modeFile : this->modeFile,
                 .modePath = param.modePath != 0 ? param.modePath : this->modePath, .user = param.user, .group = param.group,
                 .timeModified = param.timeModified, .createPath = !param.noCreatePath, .syncFile = !param.noSyncFile,
-                .syncPath = !param.noSyncPath, .atomic = !param.noAtomic, .compressible = param.compressible),
+                .syncPath = !param.noSyncPath, .atomic = !param.noAtomic, .truncate = !param.noTruncate,
+                .compressible = param.compressible),
             memContextPrior());
     }
     MEM_CONTEXT_TEMP_END();
@@ -670,7 +510,7 @@ storageNewWrite(const Storage *this, const String *fileExp, StorageNewWriteParam
 }
 
 /**********************************************************************************************************************************/
-String *
+FN_EXTERN String *
 storagePath(const Storage *this, const String *pathExp, StoragePathParam param)
 {
     FUNCTION_TEST_BEGIN();
@@ -767,11 +607,11 @@ storagePath(const Storage *this, const String *pathExp, StoragePathParam param)
         }
     }
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storagePathCreate(const Storage *this, const String *pathExp, StoragePathCreateParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -801,7 +641,7 @@ storagePathCreate(const Storage *this, const String *pathExp, StoragePathCreateP
 }
 
 /**********************************************************************************************************************************/
-bool
+FN_EXTERN bool
 storagePathExists(const Storage *this, const String *pathExp)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -820,7 +660,7 @@ storagePathExists(const Storage *this, const String *pathExp)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storagePathRemove(const Storage *this, const String *pathExp, StoragePathRemoveParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -852,7 +692,7 @@ storagePathRemove(const Storage *this, const String *pathExp, StoragePathRemoveP
 }
 
 /**********************************************************************************************************************************/
-void storagePathSync(const Storage *this, const String *pathExp)
+FN_EXTERN void storagePathSync(const Storage *this, const String *pathExp)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STORAGE, this);
@@ -876,7 +716,7 @@ void storagePathSync(const Storage *this, const String *pathExp)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storagePut(StorageWrite *file, const Buffer *buffer)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -894,7 +734,7 @@ storagePut(StorageWrite *file, const Buffer *buffer)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 storageRemove(const Storage *this, const String *fileExp, StorageRemoveParam param)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -920,10 +760,10 @@ storageRemove(const Storage *this, const String *fileExp, StorageRemoveParam par
 }
 
 /**********************************************************************************************************************************/
-String *
-storageToLog(const Storage *this)
+FN_EXTERN void
+storageToLog(const Storage *const this, StringStatic *const debugLog)
 {
-    return strNewFmt(
-        "{type: %s, path: %s, write: %s}", strZ(strIdToStr(storageType(this))), strZ(strToLog(this->path)),
-        cvtBoolToConstZ(this->write));
+    strStcCat(debugLog, "{type: "),
+    strStcResultSizeInc(debugLog, strIdToLog(storageType(this), strStcRemains(debugLog), strStcRemainsSize(debugLog)));
+    strStcFmt(debugLog, ", path: %s, write: %s}", strZ(this->path), cvtBoolToConstZ(this->write));
 }
