@@ -29,12 +29,10 @@ storageTestDummyInfo(THIS_VOID, const String *file, StorageInfoLevel level, Stor
     (void)thisVoid; (void)file; (void)level; (void)param; return (StorageInfo){.exists = false};
 }
 
-static bool
-storageTestDummyInfoList(
-    THIS_VOID, const String *path, StorageInfoLevel level, StorageInfoListCallback callback, void *callbackData,
-    StorageInterfaceInfoListParam param)
+static StorageList *
+storageTestDummyList(THIS_VOID, const String *path, StorageInfoLevel level, StorageInterfaceListParam param)
 {
-    (void)thisVoid; (void)path; (void)level; (void)callback; (void)callbackData; (void)param; return false;
+    (void)thisVoid; (void)path; (void)level; (void)param; return false;
 }
 
 static StorageRead *
@@ -64,121 +62,13 @@ storageTestDummyRemove(THIS_VOID, const String *file, StorageInterfaceRemovePara
 const StorageInterface storageInterfaceTestDummy =
 {
     .info = storageTestDummyInfo,
-    .infoList = storageTestDummyInfoList,
+    .list = storageTestDummyList,
     .newRead = storageTestDummyNewRead,
     .newWrite = storageTestDummyNewWrite,
     .pathRemove = storageTestDummyPathRemove,
     .remove = storageTestDummyRemove,
 };
 
-/**********************************************************************************************************************************/
-void
-hrnStorageInfoListCallback(void *callbackData, const StorageInfo *info)
-{
-    HarnessStorageInfoListCallbackData *data = callbackData;
-
-    if (data->rootPathOmit && info->type == storageTypePath && strEq(info->name, DOT_STR))
-        return;
-
-    strCatFmt(data->content, "%s {", info->name == NULL ? NULL_Z : strZ(info->name));
-
-    if (info->level > storageInfoLevelExists)
-    {
-        switch (info->type)
-        {
-            case storageTypeFile:
-            {
-                strCatZ(data->content, "file");
-
-                if (info->level >= storageInfoLevelBasic && !data->sizeOmit)
-                {
-                    uint64_t size = info->size;
-
-                    // If the file is compressed then decompress to get the real size.  Note that only gz is used in unit tests since
-                    // it is the only compression type guaranteed to be present.
-                    if (data->fileCompressed)
-                    {
-                        ASSERT(data->storage != NULL);
-
-                        StorageRead *read = storageNewReadP(
-                            data->storage,
-                            data->path != NULL ? strNewFmt("%s/%s", strZ(data->path), strZ(info->name)) : info->name);
-                        ioFilterGroupAdd(ioReadFilterGroup(storageReadIo(read)), decompressFilter(compressTypeGz));
-                        size = bufUsed(storageGetP(read));
-                    }
-
-                    strCatFmt(data->content, ", s=%" PRIu64, size);
-                }
-
-                break;
-            }
-
-            case storageTypeLink:
-            {
-                strCatZ(data->content, "link");
-
-                if (info->linkDestination != NULL)
-                    strCatFmt(data->content, ", d=%s", strZ(info->linkDestination));
-
-                break;
-            }
-
-            case storageTypePath:
-                strCatZ(data->content, "path");
-                break;
-
-            case storageTypeSpecial:
-                strCatZ(data->content, "special");
-                break;
-        }
-
-        if (info->type != storageTypeSpecial)
-        {
-            if (info->type != storageTypeLink)
-            {
-                if (info->level >= storageInfoLevelDetail &&
-                    (!data->modeOmit || (info->type == storageTypePath && data->modePath != info->mode) ||
-                    (info->type == storageTypeFile && data->modeFile != info->mode)))
-                {
-                    strCatFmt(data->content, ", m=%04o", info->mode);
-                }
-            }
-
-            if (info->type == storageTypeFile && info->level >= storageInfoLevelBasic)
-            {
-                if (!data->timestampOmit)
-                    strCatFmt(data->content, ", t=%" PRIu64, (uint64_t)info->timeModified);
-            }
-
-            if (info->level >= storageInfoLevelDetail && (!data->userOmit || userId() != info->userId))
-            {
-                if (info->user != NULL)
-                {
-                    strCatFmt(data->content, ", u=%s", strZ(info->user));
-                }
-                else
-                {
-                    strCatFmt(data->content, ", u=%d", (int)info->userId);
-                }
-            }
-
-            if (info->level >= storageInfoLevelDetail && (!data->groupOmit || groupId() != info->groupId))
-            {
-                if (info->group != NULL)
-                {
-                    strCatFmt(data->content, ", g=%s", strZ(info->group));
-                }
-                else
-                {
-                    strCatFmt(data->content, ", g=%d", (int)info->groupId);
-                }
-            }
-        }
-    }
-
-    strCatZ(data->content, "}\n");
-}
-
 /**********************************************************************************************************************************/
 void
 testStorageGet(const Storage *const storage, const char *const file, const char *const expected, TestStorageGetParam param)
@@ -206,7 +96,7 @@ testStorageGet(const Storage *const storage, const char *const file, const char
         if (param.cipherPass == NULL)
             param.cipherPass = TEST_CIPHER_PASS;
 
-        ioFilterGroupAdd(filterGroup, cipherBlockNew(cipherModeDecrypt, param.cipherType, BUFSTRZ(param.cipherPass), NULL));
+        ioFilterGroupAdd(filterGroup, cipherBlockNewP(cipherModeDecrypt, param.cipherType, BUFSTRZ(param.cipherPass)));
 
         strCatFmt(filter, "enc[%s,%s] ", strZ(strIdToStr(param.cipherType)), param.cipherPass);
     }
@@ -248,93 +138,151 @@ testStorageExists(const Storage *const storage, const char *const file, const Te
 }
 
 /**********************************************************************************************************************************/
-static void
-hrnStorageListCallback(void *list, const StorageInfo *info)
-{
-    if (!strEq(info->name, DOT_STR))
-    {
-        MEM_CONTEXT_BEGIN(lstMemContext(list))
-        {
-            StorageInfo infoCopy = *info;
-            infoCopy.name = strDup(infoCopy.name);
-            infoCopy.user = strDup(infoCopy.user);
-            infoCopy.group = strDup(infoCopy.group);
-            infoCopy.linkDestination = strDup(infoCopy.linkDestination);
-
-            lstAdd(list, &infoCopy);
-        }
-        MEM_CONTEXT_END();
-    }
-}
-
 void
 hrnStorageList(const Storage *const storage, const char *const path, const char *const expected, const HrnStorageListParam param)
 {
+    // Check if paths are supported
+    const bool featurePath = storageFeature(storage, storageFeaturePath);
+
+    // Determine sort order
+    const SortOrder sortOrder = param.sortOrder == sortOrderNone ? sortOrderAsc : param.sortOrder;
+
+    // Determine level
+    StorageInfoLevel level = param.level == storageInfoLevelDefault && !param.levelForce ? storageInfoLevelType : param.level;
+
     // Log list test
     hrnTestResultBegin(__func__, false);
 
     ASSERT(storage != NULL);
-    ASSERT(storagePathExistsP(storage, path == NULL ? NULL : STR(path)));
+    ASSERT(!featurePath || storagePathExistsP(storage, path == NULL ? NULL : STR(path)));
 
     const String *const pathFull = storagePathP(storage, path == NULL ? NULL : STR(path));
     printf("list%s contents of '%s'", param.remove ? "/remove": "", strZ(pathFull));
     hrnTestResultComment(param.comment);
 
     // Generate a list of files/paths/etc
-    List *list = lstNewP(sizeof(StorageInfo));
+    StorageList *const list = storageLstNew(level == storageInfoLevelDefault ? storageInfoLevelDetail : level);
 
-    storageInfoListP(
-        storage, pathFull, hrnStorageListCallback, list, .sortOrder = sortOrderAsc, .recurse = !param.noRecurse,
+    StorageIterator *const storageItr = storageNewItrP(
+        storage, pathFull, .recurse = !param.noRecurse, .sortOrder = sortOrder, .level = level,
         .expression = param.expression != NULL ? STR(param.expression) : NULL);
 
+    while (storageItrMore(storageItr))
+    {
+        StorageInfo info = storageItrNext(storageItr);
+        storageLstAdd(list, &info);
+    }
+
     // Remove files if requested
     if (param.remove)
     {
-        for (unsigned int listIdx = 0; listIdx < lstSize(list); listIdx++)
+        for (unsigned int listIdx = 0; listIdx < storageLstSize(list); listIdx++)
         {
-            const StorageInfo *const info = lstGet(list, listIdx);
+            const StorageInfo info = storageLstGet(list, listIdx);
+
+            if (strEq(info.name, DOT_STR))
+                continue;
 
             // Only remove at the top level since path remove will recurse
-            if (strChr(info->name, '/') == -1)
+            if (strChr(info.name, '/') == -1)
             {
                 // Remove a path recursively
-                if (info->type == storageTypePath)
+                if (info.type == storageTypePath)
                 {
                     storagePathRemoveP(
-                        storage, strNewFmt("%s/%s", path, strZ(info->name)), .errorOnMissing = true, .recurse = true);
+                        storage, strNewFmt("%s/%s", strZ(pathFull), strZ(info.name)), .errorOnMissing = featurePath,
+                        .recurse = true);
                 }
                 // Remove file, link, or special
                 else
-                    storageRemoveP(storage, strNewFmt("%s/%s", path, strZ(info->name)), .errorOnMissing = true);
+                    storageRemoveP(storage, strNewFmt("%s/%s", strZ(pathFull), strZ(info.name)), .errorOnMissing = true);
             }
         }
     }
 
-    // Return list for comparison
+    // Generate list for comparison
     StringList *listStr = strLstNew();
 
-    for (unsigned int listIdx = 0; listIdx < lstSize(list); listIdx++)
-    {
-        const StorageInfo *const info = lstGet(list, listIdx);
+    if (level == storageInfoLevelDefault)
+        level = storageFeature(storage, storageFeatureInfoDetail) ? storageInfoLevelDetail : storageInfoLevelBasic;
 
-        switch (info->type)
+    if (param.includeDot)
+    {
+        StorageInfo info = storageInfoP(storage, pathFull);
+        info.name = DOT_STR;
+
+        if (sortOrder == sortOrderAsc)
+            storageLstInsert(list, 0, &info);
+        else
+            storageLstAdd(list, &info);
+    }
+
+    for (unsigned int listIdx = 0; listIdx < storageLstSize(list); listIdx++)
+    {
+        const StorageInfo info = storageLstGet(list, listIdx);
+        String *const item = strCat(strNew(), info.name);
+
+        if (strEq(info.name, DOT_STR) && !param.includeDot)
+            continue;
+
+        switch (info.type)
         {
             case storageTypeFile:
-                strLstAdd(listStr, info->name);
                 break;
 
             case storageTypeLink:
-                strLstAdd(listStr, strNewFmt("%s>", strZ(info->name)));
+                strCatZ(item, ">");
                 break;
 
             case storageTypePath:
-                strLstAdd(listStr, strNewFmt("%s/", strZ(info->name)));
+                strCatZ(item, "/");
                 break;
 
             case storageTypeSpecial:
-                strLstAdd(listStr, strNewFmt("%s*", strZ(info->name)));
+                strCatZ(item, "*");
                 break;
         }
+
+        if (((info.type == storageTypeFile || info.type == storageTypeLink) && level >= storageInfoLevelBasic) ||
+            (info.type == storageTypePath && level >= storageInfoLevelDetail))
+        {
+            strCatZ(item, " {");
+
+            if (info.type == storageTypeFile)
+                strCatFmt(item, "s=%" PRIu64 ", t=%" PRId64, info.size, (int64_t)info.timeModified);
+            else if (info.type == storageTypeLink)
+            {
+                const StorageInfo infoLink = storageInfoP(
+                    storage,
+                    strEq(info.name, DOT_STR) ? pathFull : strNewFmt("%s/%s", strZ(pathFull), strZ(info.name)),
+                    .level = storageInfoLevelDetail);
+
+                strCatFmt(item, "d=%s", strZ(infoLink.linkDestination));
+            }
+
+            if (level >= storageInfoLevelDetail)
+            {
+                if (info.type != storageTypePath)
+                    strCatZ(item, ", ");
+
+                if (info.user != NULL)
+                    strCatFmt(item, "u=%s", strZ(info.user));
+                else
+                    strCatFmt(item, "u=%d", (int)info.userId);
+
+                if (info.group != NULL)
+                    strCatFmt(item, ", g=%s", strZ(info.group));
+                else
+                    strCatFmt(item, ", g=%d", (int)info.groupId);
+
+                if (info.type != storageTypeLink)
+                    strCatFmt(item, ", m=%04o", info.mode);
+            }
+
+            strCatZ(item, "}");
+        }
+
+        strLstAdd(listStr, item);
     }
 
     hrnTestResultStringList(listStr, expected, harnessTestResultOperationEq);
@@ -467,7 +415,7 @@ hrnStoragePut(
         if (param.cipherPass == NULL)
             param.cipherPass = TEST_CIPHER_PASS;
 
-        ioFilterGroupAdd(filterGroup, cipherBlockNew(cipherModeEncrypt, param.cipherType, BUFSTRZ(param.cipherPass), NULL));
+        ioFilterGroupAdd(filterGroup, cipherBlockNewP(cipherModeEncrypt, param.cipherType, BUFSTRZ(param.cipherPass)));
     }
 
     // Add file name
