@@ -29,6 +29,8 @@ typedef struct TestRequestParam
     const char *securityToken;
     const char *range;
     const char *kms;
+    const char *ttl;
+    const char *token;
 } TestRequestParam;
 
 #define testRequestP(write, s3, verb, path, ...)                                                                                   \
@@ -89,17 +91,16 @@ testRequest(IoWrite *write, Storage *s3, const char *verb, const char *path, Tes
     if (param.content != NULL)
     {
         strCatFmt(
-            request, "content-md5:%s\r\n",
-            strZ(strNewEncode(encodeBase64, cryptoHashOne(HASH_TYPE_MD5_STR, BUFSTRZ(param.content)))));
+            request, "content-md5:%s\r\n", strZ(strNewEncode(encodingBase64, cryptoHashOne(hashTypeMd5, BUFSTRZ(param.content)))));
     }
 
     // Add host
     if (s3 != NULL)
     {
         if (driver->uriStyle == storageS3UriStyleHost)
-            strCatFmt(request, "host:bucket." S3_TEST_HOST "\r\n");
+            strCatZ(request, "host:bucket." S3_TEST_HOST "\r\n");
         else
-            strCatFmt(request, "host:" S3_TEST_HOST "\r\n");
+            strCatZ(request, "host:" S3_TEST_HOST "\r\n");
     }
     else
         strCatFmt(request, "host:%s\r\n", strZ(hrnServerHost()));
@@ -116,8 +117,8 @@ testRequest(IoWrite *write, Storage *s3, const char *verb, const char *path, Tes
             request,
             "x-amz-content-sha256:%s\r\n"
                 "x-amz-date:????????T??????Z" "\r\n",
-            param.content == NULL ? HASH_TYPE_SHA256_ZERO : strZ(bufHex(cryptoHashOne(HASH_TYPE_SHA256_STR,
-            BUFSTRZ(param.content)))));
+            param.content == NULL ?
+                HASH_TYPE_SHA256_ZERO : strZ(strNewEncode(encodingHex, cryptoHashOne(hashTypeSha256, BUFSTRZ(param.content)))));
 
         // Add security token
         if (securityToken != NULL)
@@ -131,6 +132,14 @@ testRequest(IoWrite *write, Storage *s3, const char *verb, const char *path, Tes
         strCatFmt(request, "x-amz-server-side-encryption-aws-kms-key-id:%s\r\n", param.kms);
     }
 
+    // Add metadata token
+    if (param.token != NULL)
+        strCatFmt(request, "x-aws-ec2-metadata-token:%s\r\n", param.token);
+
+    // Add metadata token ttl
+    if (param.ttl != NULL)
+        strCatFmt(request, "x-aws-ec2-metadata-token-ttl-seconds:%s\r\n", param.ttl);
+
     // Add final \r\n
     strCatZ(request, "\r\n");
 
@@ -266,6 +275,8 @@ testRun(void)
     // *****************************************************************************************************************************
     if (testBegin("storageS3DateTime() and storageS3Auth()"))
     {
+        char logBuf[STACK_TRACE_PARAM_MAX];
+
         TEST_RESULT_STR_Z(storageS3DateTime(1491267845), "20170404T010405Z", "static date");
 
         // -------------------------------------------------------------------------------------------------------------------------
@@ -282,9 +293,10 @@ testRun(void)
         TEST_RESULT_STR(driver->accessKey, accessKey, "check access key");
         TEST_RESULT_STR(driver->secretAccessKey, secretAccessKey, "check secret access key");
         TEST_RESULT_STR(driver->securityToken, NULL, "check security token");
-        TEST_RESULT_STR(
-            httpClientToLog(driver->httpClient),
-            strNewFmt(
+
+        TEST_RESULT_VOID(FUNCTION_LOG_OBJECT_FORMAT(driver->httpClient, httpClientToLog, logBuf, sizeof(logBuf)), "httpClientToLog");
+        TEST_RESULT_Z(logBuf,
+            zNewFmt(
                 "{ioClient: {type: tls, driver: {ioClient: {type: socket, driver: {host: bucket.s3.amazonaws.com, port: 443"
                     ", timeoutConnect: 60000, timeoutSession: 60000}}, timeoutConnect: 60000, timeoutSession: 60000"
                     ", verifyPeer: %s}}, reusable: 0, timeout: 60000}",
@@ -299,7 +311,8 @@ testRun(void)
         httpQueryAdd(query, STRDEF("list-type"), STRDEF("2"));
 
         TEST_RESULT_VOID(
-            storageS3Auth(driver, STRDEF("GET"), STRDEF("/"), query, STRDEF("20170606T121212Z"), header, HASH_TYPE_SHA256_ZERO_STR),
+            storageS3Auth(
+                driver, STRDEF("GET"), STRDEF("/"), query, STRDEF("20170606T121212Z"), header, STRDEF(HASH_TYPE_SHA256_ZERO)),
             "generate authorization");
         TEST_RESULT_STR_Z(
             httpHeaderGet(header, STRDEF("authorization")),
@@ -312,7 +325,8 @@ testRun(void)
         const Buffer *lastSigningKey = driver->signingKey;
 
         TEST_RESULT_VOID(
-            storageS3Auth(driver, STRDEF("GET"), STRDEF("/"), query, STRDEF("20170606T121212Z"), header, HASH_TYPE_SHA256_ZERO_STR),
+            storageS3Auth(
+                driver, STRDEF("GET"), STRDEF("/"), query, STRDEF("20170606T121212Z"), header, STRDEF(HASH_TYPE_SHA256_ZERO)),
             "generate authorization");
         TEST_RESULT_STR_Z(
             httpHeaderGet(header, STRDEF("authorization")),
@@ -326,7 +340,8 @@ testRun(void)
         TEST_TITLE("change date to generate new signing key");
 
         TEST_RESULT_VOID(
-            storageS3Auth(driver, STRDEF("GET"), STRDEF("/"), query, STRDEF("20180814T080808Z"), header, HASH_TYPE_SHA256_ZERO_STR),
+            storageS3Auth(
+                driver, STRDEF("GET"), STRDEF("/"), query, STRDEF("20180814T080808Z"), header, STRDEF(HASH_TYPE_SHA256_ZERO)),
             "generate authorization");
         TEST_RESULT_STR_Z(
             httpHeaderGet(header, STRDEF("authorization")),
@@ -349,9 +364,10 @@ testRun(void)
         driver = (StorageS3 *)storageDriver(storageRepoGet(0, false));
 
         TEST_RESULT_STR(driver->securityToken, securityToken, "check security token");
-        TEST_RESULT_STR(
-            httpClientToLog(driver->httpClient),
-            strNewFmt(
+        TEST_RESULT_VOID(
+            FUNCTION_LOG_OBJECT_FORMAT(driver->httpClient, httpClientToLog, logBuf, sizeof(logBuf)), "httpClientToLog");
+        TEST_RESULT_Z(logBuf,
+            zNewFmt(
                 "{ioClient: {type: tls, driver: {ioClient: {type: socket, driver: {host: bucket.custom.endpoint, port: 333"
                     ", timeoutConnect: 60000, timeoutSession: 60000}}, timeoutConnect: 60000, timeoutSession: 60000"
                     ", verifyPeer: %s}}, reusable: 0, timeout: 60000}",
@@ -362,7 +378,8 @@ testRun(void)
         TEST_TITLE("auth with token");
 
         TEST_RESULT_VOID(
-            storageS3Auth(driver, STRDEF("GET"), STRDEF("/"), query, STRDEF("20170606T121212Z"), header, HASH_TYPE_SHA256_ZERO_STR),
+            storageS3Auth(
+                driver, STRDEF("GET"), STRDEF("/"), query, STRDEF("20170606T121212Z"), header, STRDEF(HASH_TYPE_SHA256_ZERO)),
             "generate authorization");
         TEST_RESULT_STR_Z(
             httpHeaderGet(header, STRDEF("authorization")),
@@ -411,6 +428,7 @@ testRun(void)
 
                 TEST_RESULT_STR(s3->path, path, "check path");
                 TEST_RESULT_BOOL(storageFeature(s3, storageFeaturePath), false, "check path feature");
+                TEST_RESULT_UINT(driver->partSize, 5 * 1024 * 1024, "check part size");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("coverage for noop functions");
@@ -490,7 +508,13 @@ testRun(void)
 
                 hrnServerScriptAccept(auth);
 
-                testRequestP(auth, NULL, HTTP_VERB_GET, S3_CREDENTIAL_PATH);
+                testRequestP(auth, NULL, HTTP_VERB_PUT, "/latest/api/token", .ttl = "15");
+                testResponseP(auth, .http = "1.0", .content = "WtokenW");
+
+                hrnServerScriptClose(auth);
+                hrnServerScriptAccept(auth);
+
+                testRequestP(auth, NULL, HTTP_VERB_GET, S3_CREDENTIAL_PATH, .token = "WtokenW");
                 testResponseP(auth, .http = "1.0", .code = 301);
 
                 hrnServerScriptClose(auth);
@@ -502,14 +526,21 @@ testRun(void)
                         "GET /latest/meta-data/iam/security-credentials\n"
                         "*** Request Headers ***:\n"
                         "content-length: 0\n"
-                        "host: %s",
+                        "host: %s\n"
+                        "x-aws-ec2-metadata-token: WtokenW",
                     strZ(hrnServerHost()));
 
                 // -----------------------------------------------------------------------------------------------------------------
-                TEST_TITLE("missing role");
+                TEST_TITLE("missing role and token");
 
                 hrnServerScriptAccept(auth);
 
+                testRequestP(auth, NULL, HTTP_VERB_PUT, "/latest/api/token", .ttl = "15");
+                testResponseP(auth, .http = "1.0", .code = 301);
+
+                hrnServerScriptClose(auth);
+                hrnServerScriptAccept(auth);
+
                 testRequestP(auth, NULL, HTTP_VERB_GET, S3_CREDENTIAL_PATH);
                 testResponseP(auth, .http = "1.0", .code = 404);
 
@@ -521,17 +552,23 @@ testRun(void)
                         "HINT: is a valid IAM role associated with this instance?");
 
                 // -----------------------------------------------------------------------------------------------------------------
-                TEST_TITLE("error when retrieving temp credentials");
+                TEST_TITLE("error when retrieving temp credentials and missing token");
 
                 hrnServerScriptAccept(auth);
 
+                testRequestP(auth, NULL, HTTP_VERB_PUT, "/latest/api/token", .ttl = "15");
+                testResponseP(auth, .http = "1.0", .code = 301);
+
+                hrnServerScriptClose(auth);
+                hrnServerScriptAccept(auth);
+
                 testRequestP(auth, NULL, HTTP_VERB_GET, S3_CREDENTIAL_PATH);
                 testResponseP(auth, .http = "1.0", .content = strZ(credRole));
 
                 hrnServerScriptClose(auth);
                 hrnServerScriptAccept(auth);
 
-                testRequestP(auth, NULL, HTTP_VERB_GET, strZ(strNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole))));
+                testRequestP(auth, NULL, HTTP_VERB_GET, zNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole)));
                 testResponseP(auth, .http = "1.0", .code = 300);
 
                 hrnServerScriptClose(auth);
@@ -547,11 +584,17 @@ testRun(void)
                     strZ(hrnServerHost()));
 
                 // -----------------------------------------------------------------------------------------------------------------
-                TEST_TITLE("invalid temp credentials role");
+                TEST_TITLE("invalid temp credentials role and missing token");
 
                 hrnServerScriptAccept(auth);
 
-                testRequestP(auth, NULL, HTTP_VERB_GET, strZ(strNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole))));
+                testRequestP(auth, NULL, HTTP_VERB_PUT, "/latest/api/token", .ttl = "15");
+                testResponseP(auth, .http = "1.0", .code = 301);
+
+                hrnServerScriptClose(auth);
+                hrnServerScriptAccept(auth);
+
+                testRequestP(auth, NULL, HTTP_VERB_GET, zNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole)));
                 testResponseP(auth, .http = "1.0", .code = 404);
 
                 hrnServerScriptClose(auth);
@@ -567,7 +610,14 @@ testRun(void)
 
                 hrnServerScriptAccept(auth);
 
-                testRequestP(auth, NULL, HTTP_VERB_GET, strZ(strNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole))));
+                testRequestP(auth, NULL, HTTP_VERB_PUT, "/latest/api/token", .ttl = "15");
+                testResponseP(auth, .http = "1.0", .content = "XtokenX");
+
+                hrnServerScriptClose(auth);
+                hrnServerScriptAccept(auth);
+
+                testRequestP(
+                    auth, NULL, HTTP_VERB_GET, zNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole)), .token = "XtokenX");
                 testResponseP(auth, .http = "1.0", .content = "{\"Code\":\"IAM role is not configured\"}");
 
                 hrnServerScriptClose(auth);
@@ -581,14 +631,20 @@ testRun(void)
 
                 hrnServerScriptAccept(auth);
 
-                testRequestP(auth, NULL, HTTP_VERB_GET, strZ(strNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole))));
+                testRequestP(auth, NULL, HTTP_VERB_PUT, "/latest/api/token", .ttl = "15");
+                testResponseP(auth, .http = "1.0", .content = "YtokenY");
+
+                hrnServerScriptClose(auth);
+                hrnServerScriptAccept(auth);
+
+                testRequestP(
+                    auth, NULL, HTTP_VERB_GET, zNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole)), .token = "YtokenY");
                 testResponseP(
                     auth,
-                    .content = strZ(
-                        strNewFmt(
-                            "{\"Code\":\"Success\",\"AccessKeyId\":\"x\",\"SecretAccessKey\":\"y\",\"Token\":\"z\""
-                                ",\"Expiration\":\"%s\"}",
-                            strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC - 1))))));
+                    .content = zNewFmt(
+                        "{\"Code\":\"Success\",\"AccessKeyId\":\"x\",\"SecretAccessKey\":\"y\",\"Token\":\"z\""
+                            ",\"Expiration\":\"%s\"}",
+                        strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC - 1)))));
 
                 hrnServerScriptClose(auth);
 
@@ -627,14 +683,19 @@ testRun(void)
 
                 hrnServerScriptAccept(auth);
 
-                testRequestP(auth, NULL, HTTP_VERB_GET, strZ(strNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole))));
+                testRequestP(auth, NULL, HTTP_VERB_PUT, "/latest/api/token", .ttl = "15");
+                testResponseP(auth, .http = "1.0", .content = "ZtokenZ");
+
+                hrnServerScriptClose(auth);
+                hrnServerScriptAccept(auth);
+
+                testRequestP(auth, NULL, HTTP_VERB_GET, zNewFmt(S3_CREDENTIAL_PATH "/%s", strZ(credRole)), .token = "ZtokenZ");
                 testResponseP(
                     auth,
-                    .content = strZ(
-                        strNewFmt(
-                            "{\"Code\":\"Success\",\"AccessKeyId\":\"xx\",\"SecretAccessKey\":\"yy\",\"Token\":\"zz\""
-                                ",\"Expiration\":\"%s\"}",
-                            strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC * 2))))));
+                    .content = zNewFmt(
+                        "{\"Code\":\"Success\",\"AccessKeyId\":\"xx\",\"SecretAccessKey\":\"yy\",\"Token\":\"zz\""
+                            ",\"Expiration\":\"%s\"}",
+                        strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC * 2)))));
 
                 hrnServerScriptClose(auth);
 
@@ -657,6 +718,7 @@ testRun(void)
                 TEST_RESULT_STR_Z(storageWriteName(write), "/file.txt", "check file name");
                 TEST_RESULT_BOOL(storageWriteSyncFile(write), true, "file is synced");
                 TEST_RESULT_BOOL(storageWriteSyncPath(write), true, "path is synced");
+                TEST_RESULT_BOOL(storageWriteTruncate(write), true, "file will be truncated");
 
                 TEST_RESULT_VOID(storageWriteS3Close(write->driver), "close file again");
 
@@ -883,8 +945,7 @@ testRun(void)
                 testRequestP(auth, NULL, HTTP_VERB_GET, TEST_SERVICE_URI);
                 testResponseP(
                     auth,
-                    .content = strZ(
-                        strNewFmt(TEST_SERVICE_RESPONSE, strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC - 1))))));
+                    .content = zNewFmt(TEST_SERVICE_RESPONSE, strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC - 1)))));
 
                 hrnServerScriptClose(auth);
 
@@ -903,8 +964,7 @@ testRun(void)
                 testRequestP(auth, NULL, HTTP_VERB_GET, TEST_SERVICE_URI);
                 testResponseP(
                     auth,
-                    .content = strZ(
-                        strNewFmt(TEST_SERVICE_RESPONSE, strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC * 2))))));
+                    .content = zNewFmt(TEST_SERVICE_RESPONSE, strZ(testS3DateTime(time(NULL) + (S3_CREDENTIAL_RENEW_SEC * 2)))));
 
                 hrnServerScriptClose(auth);
 
@@ -1004,6 +1064,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>false</IsTruncated>"
                         "    <Contents>"
                         "        <Key>path/to/test_file</Key>"
                         "        <LastModified>2009-10-12T17:50:30.000Z</LastModified>"
@@ -1014,22 +1075,15 @@ testRun(void)
                         "   </CommonPrefixes>"
                         "</ListBucketResult>");
 
-                HarnessStorageInfoListCallbackData callbackData =
-                {
-                    .content = strNew(),
-                };
-
                 TEST_ERROR(
-                    storageInfoListP(s3, STRDEF("/"), hrnStorageInfoListCallback, NULL, .errorOnMissing = true),
-                    AssertError, "assertion '!param.errorOnMissing || storageFeature(this, storageFeaturePath)' failed");
+                    storageNewItrP(s3, STRDEF("/"), .errorOnMissing = true), AssertError,
+                    "assertion '!param.errorOnMissing || storageFeature(this, storageFeaturePath)' failed");
 
-                TEST_RESULT_VOID(
-                    storageInfoListP(s3, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData), "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test_path {path}\n"
-                    "test_file {file, s=787, t=1255369830}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    s3, "/path/to",
+                    "test_file {s=787, t=1255369830}\n"
+                    "test_path/\n",
+                    .level = storageInfoLevelBasic, .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list exists level");
@@ -1040,6 +1094,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>false</IsTruncated>"
                         "    <Contents>"
                         "        <Key>test1.txt</Key>"
                         "    </Contents>"
@@ -1048,16 +1103,11 @@ testRun(void)
                         "   </CommonPrefixes>"
                         "</ListBucketResult>");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(s3, STRDEF("/"), hrnStorageInfoListCallback, &callbackData, .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "path1 {}\n"
-                    "test1.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    s3, "/",
+                    "path1/\n"
+                    "test1.txt\n",
+                    .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list a file in root with expression");
@@ -1068,22 +1118,16 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>false</IsTruncated>"
                         "    <Contents>"
                         "        <Key>test1.txt</Key>"
                         "    </Contents>"
                         "</ListBucketResult>");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        s3, STRDEF("/"), hrnStorageInfoListCallback, &callbackData, .expression = STRDEF("^test.*$"),
-                        .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test1.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    s3, "/",
+                    "test1.txt\n",
+                    .noRecurse = true, .expression = "^test.*$");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list files with continuation");
@@ -1094,6 +1138,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>true</IsTruncated>"
                         "    <NextContinuationToken>1ueGcxLPRx1Tr/XYExHnhbYLgveDs2J/wm36Hy4vbOwM=</NextContinuationToken>"
                         "    <Contents>"
                         "        <Key>path/to/test1.txt</Key>"
@@ -1115,6 +1160,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>false</IsTruncated>"
                         "    <Contents>"
                         "        <Key>path/to/test3.txt</Key>"
                         "    </Contents>"
@@ -1123,20 +1169,14 @@ testRun(void)
                         "   </CommonPrefixes>"
                         "</ListBucketResult>");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        s3, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData, .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "path1 {}\n"
-                    "test1.txt {}\n"
-                    "test2.txt {}\n"
-                    "path2 {}\n"
-                    "test3.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    s3, "/path/to",
+                    "path1/\n"
+                    "path2/\n"
+                    "test1.txt\n"
+                    "test2.txt\n"
+                    "test3.txt\n",
+                    .noRecurse = true);
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("list files with expression");
@@ -1147,6 +1187,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>false</IsTruncated>"
                         "    <Contents>"
                         "        <Key>path/to/test1.txt</Key>"
                         "    </Contents>"
@@ -1164,19 +1205,12 @@ testRun(void)
                         "   </CommonPrefixes>"
                         "</ListBucketResult>");
 
-                callbackData.content = strNew();
-
-                TEST_RESULT_VOID(
-                    storageInfoListP(
-                        s3, STRDEF("/path/to"), hrnStorageInfoListCallback, &callbackData, .expression = STRDEF("^test(1|3)"),
-                        .level = storageInfoLevelExists),
-                    "list");
-                TEST_RESULT_STR_Z(
-                    callbackData.content,
-                    "test1.path {}\n"
-                    "test1.txt {}\n"
-                    "test3.txt {}\n",
-                    "check");
+                TEST_STORAGE_LIST(
+                    s3, "/path/to",
+                    "test1.path\n"
+                    "test1.txt\n"
+                    "test3.txt\n",
+                    .level = storageInfoLevelExists, .noRecurse = true, .expression = "^test(1|3)");
 
                 // -----------------------------------------------------------------------------------------------------------------
                 TEST_TITLE("switch to path-style URIs");
@@ -1214,6 +1248,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>false</IsTruncated>"
                         "    <Contents>"
                         "        <Key>test1.txt</Key>"
                         "    </Contents>"
@@ -1244,6 +1279,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>false</IsTruncated>"
                         "</ListBucketResult>");
 
                 TEST_RESULT_VOID(storagePathRemoveP(s3, STRDEF("/path"), .recurse = true), "remove");
@@ -1257,6 +1293,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>true</IsTruncated>"
                         "    <NextContinuationToken>continue</NextContinuationToken>"
                         "   <CommonPrefixes>"
                         "       <Prefix>path/to/test3/</Prefix>"
@@ -1272,6 +1309,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>false</IsTruncated>"
                         "    <Contents>"
                         "        <Key>path/to/test3.txt</Key>"
                         "    </Contents>"
@@ -1310,6 +1348,7 @@ testRun(void)
                     .content =
                         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                         "<ListBucketResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">"
+                        "    <IsTruncated>false</IsTruncated>"
                         "    <Contents>"
                         "        <Key>path/sample.txt</Key>"
                         "    </Contents>"
